@@ -226,7 +226,7 @@ void check_save_to_file() {
   ss << "config.logging_level 0\n";
 #endif
   ss << "filestore.buffer_size 104857600\n";
-  ss << "rest.capnp_traversal_limit 536870912\n";
+  ss << "rest.capnp_traversal_limit 2147483648\n";
   ss << "rest.curl.buffer_size 524288\n";
   ss << "rest.curl.verbose false\n";
   ss << "rest.http_compressor any\n";
@@ -239,8 +239,8 @@ void check_save_to_file() {
   ss << "rest.retry_initial_delay_ms 500\n";
   ss << "rest.server_address https://api.tiledb.com\n";
   ss << "rest.server_serialization_format CAPNP\n";
-  ss << "rest.use_refactored_array_open false\n";
-  ss << "rest.use_refactored_array_open_and_query_submit false\n";
+  ss << "rest.use_refactored_array_open true\n";
+  ss << "rest.use_refactored_array_open_and_query_submit true\n";
   ss << "sm.allow_separate_attribute_writes false\n";
   ss << "sm.allow_updates_experimental false\n";
   ss << "sm.check_coord_dups true\n";
@@ -579,13 +579,13 @@ TEST_CASE("C API: Test config iter", "[capi][config]") {
   CHECK(rc == TILEDB_OK);
   CHECK(error == nullptr);
   rc = tiledb_config_set(
-      config, "rest.use_refactored_array_open", "true", &error);
+      config, "rest.use_refactored_array_open", "false", &error);
   CHECK(rc == TILEDB_OK);
   CHECK(error == nullptr);
   rc = tiledb_config_set(
       config,
       "rest.use_refactored_array_open_and_query_submit",
-      "true",
+      "false",
       &error);
   CHECK(rc == TILEDB_OK);
   CHECK(error == nullptr);
@@ -607,14 +607,14 @@ TEST_CASE("C API: Test config iter", "[capi][config]") {
   all_param_values["rest.retry_delay_factor"] = "1.25";
   all_param_values["rest.retry_initial_delay_ms"] = "500";
   all_param_values["rest.retry_http_codes"] = "503";
-  all_param_values["rest.capnp_traversal_limit"] = "536870912";
+  all_param_values["rest.capnp_traversal_limit"] = "2147483648";
   all_param_values["rest.curl.buffer_size"] = "524288";
   all_param_values["rest.curl.verbose"] = "false";
   all_param_values["rest.load_metadata_on_array_open"] = "false";
   all_param_values["rest.load_non_empty_domain_on_array_open"] = "false";
   all_param_values["rest.load_enumerations_on_array_open"] = "false";
-  all_param_values["rest.use_refactored_array_open"] = "true";
-  all_param_values["rest.use_refactored_array_open_and_query_submit"] = "true";
+  all_param_values["rest.use_refactored_array_open"] = "false";
+  all_param_values["rest.use_refactored_array_open_and_query_submit"] = "false";
   all_param_values["rest.payer_namespace"] = "";
   all_param_values["sm.allow_separate_attribute_writes"] = "false";
   all_param_values["sm.allow_updates_experimental"] = "false";
