@@ -85,7 +85,7 @@ struct GroupFx {
   std::string get_golden_walk(const std::string& path);
   std::string get_golden_ls(const std::string& path);
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> read_group(
-      tiledb_group_t* group, bool use_get_member_by_index_v2 = true) const;
+      tiledb_group_t* group) const;
   void set_group_timestamp(
       tiledb_group_t* group, const uint64_t& timestamp) const;
   void write_group_metadata(const char* group_uri) const;
@@ -229,49 +229,30 @@ void GroupFx::vacuum(const char* group_uri) const {
 }
 
 std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> GroupFx::read_group(
-    tiledb_group_t* group, bool use_get_member_by_index_v2) const {
+    tiledb_group_t* group) const {
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> ret;
   uint64_t count = 0;
   int rc = tiledb_group_get_member_count(ctx_, group, &count);
   REQUIRE(rc == TILEDB_OK);
-  if (use_get_member_by_index_v2) {
-    for (uint64_t i = 0; i < count; i++) {
-      std::string uri;
-      tiledb_object_t type;
-      tiledb_string_t *uri_handle, *name_handle;
-      rc = tiledb_group_get_member_by_index_v2(
-          ctx_, group, i, &uri_handle, &type, &name_handle);
-      REQUIRE(rc == TILEDB_OK);
-      const char* uri_str;
-      size_t uri_size;
-      rc = tiledb_string_view(uri_handle, &uri_str, &uri_size);
+  for (uint64_t i = 0; i < count; i++) {
+    std::string uri;
+    tiledb_object_t type;
+    tiledb_string_t *uri_handle, *name_handle;
+    rc = tiledb_group_get_member_by_index_v2(
+        ctx_, group, i, &uri_handle, &type, &name_handle);
+    REQUIRE(rc == TILEDB_OK);
+    const char* uri_str;
+    size_t uri_size;
+    rc = tiledb_string_view(uri_handle, &uri_str, &uri_size);
+    CHECK(rc == TILEDB_OK);
+    uri = std::string(uri_str, uri_size);
+    rc = tiledb_string_free(&uri_handle);
+    CHECK(rc == TILEDB_OK);
+    if (name_handle) {
+      rc = tiledb_string_free(&name_handle);
       CHECK(rc == TILEDB_OK);
-      uri = std::string(uri_str, uri_size);
-      rc = tiledb_string_free(&uri_handle);
-      CHECK(rc == TILEDB_OK);
-      if (name_handle) {
-        rc = tiledb_string_free(&name_handle);
-        CHECK(rc == TILEDB_OK);
-      }
-      ret.emplace_back(uri, type);
-    }
-  } else {
-    // When tiledb_group_get_member_by_index gets removed, the else part can
-    // simply be removed.
-    for (uint64_t i = 0; i < count; i++) {
-      std::string uri;
-      tiledb_object_t type;
-      char *uri_ptr, *name_ptr;
-      rc = tiledb_group_get_member_by_index(
-          ctx_, group, i, &uri_ptr, &type, &name_ptr);
-      REQUIRE(rc == TILEDB_OK);
-      uri = uri_ptr == nullptr ? "" : std::string(uri_ptr);
-      std::free(uri_ptr);
-      if (name_ptr) {
-        std::free(name_ptr);
-      }
-      ret.emplace_back(uri, type);
     }
+    ret.emplace_back(uri, type);
   }
   return ret;
 }
@@ -415,12 +396,10 @@ TEST_CASE_METHOD(
     "[capi][group][metadata][rest][regression][sc-4821]") {
   // Create and open group in write mode
   // TODO: refactor for each supported FS.
-  std::string fs_temp_dir = fs_vec_[0]->temp_dir();
-  create_temp_dir(fs_temp_dir);
-  std::string temp_dir =
-      fs_vec_[0]->is_rest() ? "tiledb://unit/" + fs_temp_dir : fs_temp_dir;
+  std::string temp_dir = fs_vec_[0]->temp_dir();
+  create_temp_dir(temp_dir);
+  std::string group1_uri = vfs_array_uri(fs_vec_[0], temp_dir + "group1");
 
-  std::string group1_uri = temp_dir + "group1";
   REQUIRE(tiledb_group_create(ctx_, group1_uri.c_str()) == TILEDB_OK);
 
   tiledb_group_t* group;
@@ -512,7 +491,7 @@ TEST_CASE_METHOD(
   rc = tiledb_group_close(ctx_, group);
   REQUIRE(rc == TILEDB_OK);
   tiledb_group_free(&group);
-  remove_temp_dir(fs_temp_dir);
+  remove_temp_dir(temp_dir);
 }
 
 TEST_CASE_METHOD(
@@ -572,13 +551,6 @@ TEST_CASE_METHOD(
 
 TEST_CASE_METHOD(
     GroupFx, "C API: Group, write/read", "[capi][group][metadata][read]") {
-  bool use_get_member_by_index_v2 = true;
-  SECTION("Using tiledb_group_get_member_by_index_v2") {
-    use_get_member_by_index_v2 = true;
-  }
-  SECTION("Using tiledb_group_get_member_by_index") {
-    use_get_member_by_index_v2 = false;
-  }
   // Create and open group in write mode
   // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
@@ -648,12 +620,12 @@ TEST_CASE_METHOD(
   REQUIRE(rc == TILEDB_OK);
 
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> group1_received =
-      read_group(group1, use_get_member_by_index_v2);
+      read_group(group1);
   REQUIRE_THAT(
       group1_received, Catch::Matchers::UnorderedEquals(group1_expected));
 
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> group2_received =
-      read_group(group2, use_get_member_by_index_v2);
+      read_group(group2);
   REQUIRE_THAT(
       group2_received, Catch::Matchers::UnorderedEquals(group2_expected));
 
@@ -1492,16 +1464,18 @@ TEST_CASE_METHOD(
   remove_temp_dir(temp_dir);
 }
 
-TEST_CASE_METHOD(GroupFx, "C API: Group, dump", "[capi][group][dump]") {
+TEST_CASE_METHOD(
+    GroupFx,
+    "C API: Group, dump",
+    "[capi][group][dump][rest-fails][sc-48099]") {
   // Create and open group in write mode
-  // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
   create_temp_dir(temp_dir);
 
-  tiledb::sm::URI group1_uri(temp_dir + "group1");
+  std::string group1_uri = vfs_array_uri(fs_vec_[0], temp_dir + "group1");
   REQUIRE(tiledb_group_create(ctx_, group1_uri.c_str()) == TILEDB_OK);
 
-  tiledb::sm::URI group2_uri(temp_dir + "group2");
+  std::string group2_uri = vfs_array_uri(fs_vec_[0], temp_dir + "group2");
   REQUIRE(tiledb_group_create(ctx_, group2_uri.c_str()) == TILEDB_OK);
 
   tiledb_group_t* group1;
