@@ -32,6 +32,7 @@
 
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
+#include "test/support/src/vfs_helpers.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/misc/utils.h"
 
@@ -44,10 +45,6 @@ using namespace tiledb::test;
 
 class NullableArrayCppFx {
  public:
-  // Serialization parameters
-  bool serialize_ = false;
-  bool refactored_query_v2_ = false;
-
   template <typename T>
   struct test_dim_t {
     test_dim_t(
@@ -113,12 +110,6 @@ class NullableArrayCppFx {
     vector<uint8_t>* validity_bytemap_;
   };
 
-  /** Constructor. */
-  NullableArrayCppFx();
-
-  /** Destructor. */
-  ~NullableArrayCppFx();
-
   /**
    * Creates, writes, and reads nullable attributes.
    *
@@ -140,19 +131,6 @@ class NullableArrayCppFx {
   /** The C++ API context object. */
   Context ctx_;
 
-  /** The C++ API VFS object. */
-  VFS vfs_;
-
-  // Buffers to allocate on server side for serialized queries
-  ServerQueryBuffers server_buffers_;
-
-  /**
-   * Removes a directory using `vfs_`.
-   *
-   * @param path The directory path.
-   */
-  void remove_dir(const string& path);
-
   /**
    * Creates a TileDB array.
    *
@@ -199,18 +177,6 @@ class NullableArrayCppFx {
       const vector<uint64_t>& subarray);
 };
 
-NullableArrayCppFx::NullableArrayCppFx()
-    : vfs_(ctx_) {
-}
-
-NullableArrayCppFx::~NullableArrayCppFx() {
-}
-
-void NullableArrayCppFx::remove_dir(const string& path) {
-  if (vfs_.is_dir(path))
-    vfs_.remove_dir(path);
-}
-
 template <typename DIM_T, typename ATTR_T>
 void NullableArrayCppFx::create_array(
     const string& array_name,
@@ -219,8 +185,6 @@ void NullableArrayCppFx::create_array(
     const vector<test_attr_t<ATTR_T>>& test_attrs,
     const tiledb_layout_t cell_order,
     const tiledb_layout_t tile_order) {
-  remove_dir(array_name);
-
   // Create the domain.
   Domain domain(ctx_);
 
@@ -298,14 +262,11 @@ void NullableArrayCppFx::write(
   }
 
   // Submit query
-  auto rc = submit_query_wrapper(
-      ctx_,
-      array_name,
-      &query,
-      server_buffers_,
-      serialize_,
-      refactored_query_v2_);
-  REQUIRE(rc == TILEDB_OK);
+  if (layout == TILEDB_GLOBAL_ORDER) {
+    query.submit_and_finalize();
+  } else {
+    query.submit();
+  }
 
   // Clean up
   array.close();
@@ -353,7 +314,9 @@ void NullableArrayCppFx::read(
   }
 
   // Set the subarray to read.
-  query.set_subarray(subarray);
+  Subarray sub(ctx_, array);
+  sub.set_subarray(subarray);
+  query.set_subarray(sub);
 
   // Submit the query.
   REQUIRE(query.submit() == Query::Status::COMPLETE);
@@ -372,7 +335,9 @@ void NullableArrayCppFx::do_2d_nullable_test(
     const tiledb_layout_t cell_order,
     const tiledb_layout_t tile_order,
     const tiledb_layout_t write_order) {
-  const string array_name = "cpp_2d_nullable_array";
+  VFSTestSetup vfs_test_setup{};
+  ctx_ = vfs_test_setup.ctx();
+  auto array_name{vfs_test_setup.array_uri("cpp_2d_nullable_array")};
 
   // Skip row-major and col-major writes for sparse arrays.
   if (array_type == TILEDB_SPARSE &&
@@ -552,29 +517,16 @@ void NullableArrayCppFx::do_2d_nullable_test(
         expected_a3_read_buffer_validity.data(),
         a3_read_buffer_validity.size()));
   }
-
-  remove_dir(array_name);
 }
 
 TEST_CASE_METHOD(
     NullableArrayCppFx,
     "C++ API: Test 2D array with nullable attributes",
-    "[cppapi][2d][nullable]") {
+    "[cppapi][2d][nullable][rest]") {
   vector<test_attr_t<uint64_t>> attrs;
   attrs.emplace_back("a1", false /* var */, true /* nullable */);
   attrs.emplace_back("a2", false /* var */, true /* nullable */);
   attrs.emplace_back("a3", true /* var */, true /* nullable */);
-
-  SECTION("no serialization") {
-    serialize_ = false;
-  }
-  SECTION("serialization enabled global order write") {
-#ifdef TILEDB_SERIALIZATION
-    serialize_ = true;
-    refactored_query_v2_ = GENERATE(true, false);
-#endif
-  }
-
   for (auto attr_iter = attrs.begin(); attr_iter != attrs.end(); ++attr_iter) {
     vector<test_attr_t<uint64_t>> test_attrs(attrs.begin(), attr_iter + 1);
     for (const tiledb_array_type_t array_type : {TILEDB_DENSE, TILEDB_SPARSE}) {
