@@ -33,6 +33,7 @@
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
 #include "test/support/src/serialization_wrappers.h"
+#include "test/support/src/vfs_helpers.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/misc/utils.h"
@@ -79,8 +80,10 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range[] = {0, 0};
-    query.add_range(0, range[0], range[1]);
-    query.add_range(1, range[0], range[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range[0], range[1]);
+    subarray.add_range(1, range[0], range[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 4);
@@ -117,7 +120,9 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range[] = {1, 2};
-    query.add_range(0, range[0], range[1]).add_range(1, range[0], range[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range[0], range[1]).add_range(1, range[0], range[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 4);
@@ -157,27 +162,29 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range0[] = {0, 0}, range1[] = {2, 2};
-    query.add_range(0, range0[0], range0[1]);
-    query.add_range(1, range0[0], range0[1]);
-    query.add_range(0, range1[0], range1[1]);
-    query.add_range(1, range1[0], range1[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range0[0], range0[1]);
+    subarray.add_range(1, range0[0], range0[1]);
+    subarray.add_range(0, range1[0], range1[1]);
+    subarray.add_range(1, range1[0], range1[1]);
 
     int64_t inv_range[] = {0, 1};
-    CHECK_THROWS(query.add_range(1, inv_range[0], inv_range[1]));
+    CHECK_THROWS(subarray.add_range(1, inv_range[0], inv_range[1]));
+    query.set_subarray(subarray);
 
     // Get range
-    auto range = query.range<int>(0, 0);
+    auto range = subarray.range<int>(0, 0);
     CHECK(range[0] == 0);
     CHECK(range[1] == 0);
     CHECK(range[2] == 0);
-    range = query.range<int>(1, 1);
+    range = subarray.range<int>(1, 1);
     CHECK(range[0] == 2);
     CHECK(range[1] == 2);
     CHECK(range[2] == 0);
 
-    CHECK_THROWS(range = query.range<int>(1, 3));
+    CHECK_THROWS(range = subarray.range<int>(1, 3));
     std::array<int64_t, 3> range2 = {{0, 0, 0}};
-    CHECK_THROWS(range2 = query.range<int64_t>(1, 1));
+    CHECK_THROWS(range2 = subarray.range<int64_t>(1, 1));
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 4);
@@ -234,10 +241,12 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range0[] = {0, 1}, range1[] = {2, 3};
-    query.add_range(0, range0[0], range0[1]);
-    query.add_range(1, range0[0], range0[1]);
-    query.add_range(0, range1[0], range1[1]);
-    query.add_range(1, range1[0], range1[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range0[0], range0[1]);
+    subarray.add_range(1, range0[0], range0[1]);
+    subarray.add_range(0, range1[0], range1[1]);
+    subarray.add_range(1, range1[0], range1[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     std::vector<int> data(est_size);
@@ -281,8 +290,10 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     query.set_config(config);
     int range[] = {1, 4};
     int range2[] = {-1, 3};
-    REQUIRE_THROWS(query.add_range(0, range[0], range[1]));
-    REQUIRE_THROWS(query.add_range(1, range2[0], range2[1]));
+    Subarray subarray(ctx, array);
+    REQUIRE_THROWS(subarray.add_range(0, range[0], range[1]));
+    REQUIRE_THROWS(subarray.add_range(1, range2[0], range2[1]));
+    query.set_subarray(subarray);
   }
 
   SECTION("- Read ranges oob error - Subarray-cppapi") {
@@ -318,8 +329,11 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     query.set_config(config);
     int range[] = {1, 4};
     int range2[] = {-1, 3};
-    query.add_range(0, range[0], range[1]);
-    query.add_range(1, range2[0], range2[1]);
+    Subarray subarray(ctx, array);
+    subarray.set_config(config);
+    subarray.add_range(0, range[0], range[1]);
+    subarray.add_range(1, range2[0], range2[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 12);
@@ -334,18 +348,6 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     REQUIRE(data[1] == 3);
   }
 
-  SECTION("- set_subarray  Write ranges oob warn - Subarray-query") {
-    tiledb::Array array(ctx, array_name, TILEDB_WRITE);
-    tiledb::Query query(ctx, array);
-    // throws on sparse array (set_subarray not supported)
-    REQUIRE_THROWS(query.set_subarray({1, 4, -1, 3}));
-    tiledb::Config config;
-    config.set("sm.read_range_oob", "warn");
-    query.set_config(config);
-    // throws on sparse array
-    REQUIRE_THROWS(query.set_subarray({1, 4, -1, 3}));
-  }
-
   SECTION("- Read ranges oob warn - Subarray-cppapi") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Subarray subarray(ctx, array);
@@ -423,23 +425,23 @@ TEST_CASE("C++ API: Test subarray (dense)", "[cppapi][dense][subarray]") {
   }
 }
 
-#ifdef TILEDB_SERIALIZATION
 TEST_CASE(
     "C++ API: Test serialized OOB subarray error (dense)",
-    "[cppapi][dense][subarray][oob][serialization]") {
-  bool query_v2 = GENERATE(true, false);
-
-  std::string array_name = "cpp_unit_array";
+    "[cppapi][dense][subarray][oob][serialization][rest]") {
+  auto reader_type = GENERATE("legacy", "refactored");
   tiledb::Config cfg;
-  cfg.set("config.logging_level", "3");
-  cfg["sm.query.dense.reader"] = GENERATE("legacy", "refactored");
-  tiledb::Context ctx(cfg);
+  cfg["sm.query.dense.reader"] = reader_type;
+  tiledb::test::VFSTestSetup vfs_test_setup(cfg.ptr().get());
 
-  VFS vfs(ctx);
-  if (vfs.is_dir(array_name)) {
-    vfs.remove_dir(array_name);
+  // SC-45976 : Different behaviour between local and remote arrays when
+  // submitting query with ranges oob of domain
+  if (!vfs_test_setup.is_rest()) {
+    return;
   }
 
+  tiledb::Context ctx(vfs_test_setup.ctx());
+  std::string array_name = vfs_test_setup.array_uri("cpp_unit_array");
+
   // Create
   tiledb::Domain domain(ctx);
   domain.add_dimension(tiledb::Dimension::create<int>(ctx, "rows", {{0, 3}}, 4))
@@ -499,21 +501,14 @@ TEST_CASE(
 
   std::vector<int> a(4);
   query.set_data_buffer("a", a);
-  tiledb::test::ServerQueryBuffers buffers;
-  CHECK(
-      submit_query_wrapper(
-          ctx, array_name, &query, buffers, true, query_v2, false) == expected);
-
   if (expected == TILEDB_OK) {
+    CHECK_NOTHROW(query.submit());
     CHECK(query.query_status() == tiledb::Query::Status::COMPLETE);
     CHECK(a == std::vector<int>{1, 2, 3, 4});
-  }
-
-  if (vfs.is_dir(array_name)) {
-    vfs.remove_dir(array_name);
+  } else {
+    CHECK_THROWS(query.submit());
   }
 }
-#endif
 
 TEST_CASE(
     "C++ API: Test subarray (incomplete) - Subarray-query",
@@ -578,14 +573,16 @@ TEST_CASE(
   int row_range[] = {0, 1};
   int col_range0[] = {12277, 13499};
   int col_range1[] = {13500, 17486};
-  query.add_range(0, row_range[0], row_range[1]);
-  query.add_range(1, col_range0[0], col_range0[1]);
-  query.add_range(1, col_range1[0], col_range1[1]);
+  Subarray subarray(ctx, array);
+  subarray.add_range(0, row_range[0], row_range[1]);
+  subarray.add_range(1, col_range0[0], col_range0[1]);
+  subarray.add_range(1, col_range1[0], col_range1[1]);
+  query.set_subarray(subarray);
 
   // Test range num
-  auto range_num = query.range_num(0);
+  auto range_num = subarray.range_num(0);
   CHECK(range_num == 1);
-  range_num = query.range_num(1);
+  range_num = subarray.range_num(1);
   // Ranges `col_range0` and `col_range1` are coalesced.
   CHECK(range_num == 1);
 
@@ -1040,9 +1037,11 @@ TEST_CASE(
   int row_range[] = {1, 1};
   int col_range0[] = {12277, 12277};
   int col_range1[] = {12277, 13160};
-  query.add_range(0, row_range[0], row_range[1]);
-  query.add_range(1, col_range0[0], col_range0[1]);
-  query.add_range(1, col_range1[0], col_range1[1]);
+  Subarray subarray(ctx, array);
+  subarray.add_range(0, row_range[0], row_range[1]);
+  subarray.add_range(1, col_range0[0], col_range0[1]);
+  subarray.add_range(1, col_range1[0], col_range1[1]);
+  query.set_subarray(subarray);
   query.set_layout(TILEDB_UNORDERED);
 
   // Allocate buffers large enough to hold 2 cells at a time.
