@@ -1093,7 +1093,7 @@ TEST_CASE_METHOD(
     EnumerationFx,
     "Array - Get Enumeration Error - Not Open",
     "[enumeration][array][error][not-open]") {
-  auto array = make_shared<Array>(HERE(), uri_, ctx_.storage_manager());
+  auto array = make_shared<Array>(HERE(), ctx_.resources(), uri_);
   auto matcher = Catch::Matchers::ContainsSubstring("Array is not open");
   REQUIRE_THROWS(array->get_enumeration("foo"), matcher);
 }
@@ -1140,7 +1140,7 @@ TEST_CASE_METHOD(
     EnumerationFx,
     "Array - Load All Enumerations Error - Not Open",
     "[enumeration][array][error][not-open]") {
-  auto array = make_shared<Array>(HERE(), uri_, ctx_.storage_manager());
+  auto array = make_shared<Array>(HERE(), ctx_.resources(), uri_);
   auto matcher = Catch::Matchers::ContainsSubstring("Array is not open");
   REQUIRE_THROWS(array->load_all_enumerations());
 }
@@ -2064,9 +2064,11 @@ TEST_CASE_METHOD(
 
   auto ase = make_shared<ArraySchemaEvolution>(HERE(), memory_tracker_);
   ase->extend_enumeration(new_enmr);
-  auto st = ctx_.storage_manager()->array_evolve_schema(
-      array->array_uri(), ase.get(), array->get_encryption_key());
-  throw_if_not_ok(st);
+  tiledb::sm::Array::evolve_array_schema(
+      ctx_.resources(),
+      array->array_uri(),
+      ase.get(),
+      array->get_encryption_key());
 
   // Check that we can not rewrite the query condition.
   array = get_array(QueryType::READ);
@@ -2396,10 +2398,10 @@ TEST_CASE_METHOD(
   auto qc1 = create_qc("attr1", (int)2, QueryConditionOp::EQ);
   qc1.set_use_enumeration(false);
 
-  Query q1(ctx_.storage_manager(), array);
+  Query q1(ctx_.resources(), ctx_.storage_manager(), array);
   throw_if_not_ok(q1.set_condition(qc1));
 
-  Query q2(ctx_.storage_manager(), array);
+  Query q2(ctx_.resources(), ctx_.storage_manager(), array);
   ser_des_query(&q1, &q2, client_side, ser_type);
 
   auto qc2 = q2.condition();
@@ -2432,10 +2434,10 @@ TEST_CASE_METHOD(
 
   throw_if_not_ok(qc1.combine(qc2, QueryConditionCombinationOp::OR, &qc3));
 
-  Query q1(ctx_.storage_manager(), array);
+  Query q1(ctx_.resources(), ctx_.storage_manager(), array);
   throw_if_not_ok(q1.set_condition(qc3));
 
-  Query q2(ctx_.storage_manager(), array);
+  Query q2(ctx_.resources(), ctx_.storage_manager(), array);
   ser_des_query(&q1, &q2, client_side, ser_type);
 
   auto qc4 = q2.condition();
@@ -2462,7 +2464,7 @@ TEST_CASE_METHOD(
   throw_if_not_ok(cfg.set("rest.load_enumerations_on_array_open", do_load));
   Context ctx(cfg);
 
-  auto a1 = make_shared<Array>(HERE(), uri_, ctx.storage_manager());
+  auto a1 = make_shared<Array>(HERE(), ctx.resources(), uri_);
   throw_if_not_ok(
       a1->open(QueryType::READ, EncryptionType::NO_ENCRYPTION, nullptr, 0));
   REQUIRE(a1->serialize_enumerations() == (do_load == "true"));
@@ -2470,7 +2472,7 @@ TEST_CASE_METHOD(
       a1->array_schema_latest_ptr()->get_loaded_enumeration_names().size() ==
       0);
 
-  auto a2 = make_shared<Array>(HERE(), uri_, ctx.storage_manager());
+  auto a2 = make_shared<Array>(HERE(), ctx.resources(), uri_);
 
   ser_des_array(ctx, a1.get(), a2.get(), client_side, ser_type);
 
@@ -2879,11 +2881,11 @@ shared_ptr<ArraySchema> EnumerationFx::create_schema() {
 
 void EnumerationFx::create_array() {
   auto schema = create_schema();
-  throw_if_not_ok(ctx_.storage_manager()->array_create(uri_, schema, enc_key_));
+  Array::create(ctx_.resources(), uri_, schema, enc_key_);
 }
 
 shared_ptr<Array> EnumerationFx::get_array(QueryType type) {
-  auto array = make_shared<Array>(HERE(), uri_, ctx_.storage_manager());
+  auto array = make_shared<Array>(HERE(), ctx_.resources(), uri_);
   throw_if_not_ok(array->open(type, EncryptionType::NO_ENCRYPTION, nullptr, 0));
   return array;
 }
