@@ -35,7 +35,7 @@
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
 #include "test/support/src/vfs_helpers.h"
-#include "tiledb/common/thread_pool.h"
+#include "tiledb/common/thread_pool/thread_pool.h"
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/filesystem/s3.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
@@ -124,7 +124,7 @@ TEST_CASE_METHOD(S3Fx, "Test S3 multiupload abort path", "[s3]") {
     CHECK(!exists);
 
     // Flush the file
-    CHECK(s3_.flush_object(URI(largefile)).ok());
+    CHECK(!s3_.flush_object(URI(largefile)).ok());
 
     // After flushing, the file does not exist
     CHECK(s3_.is_object(URI(largefile), &exists).ok());
