@@ -339,9 +339,7 @@ TEMPLATE_LIST_TEST_CASE(
     std::string s = "abcdef";
     require_tiledb_ok(vfs.write(ls_file, s.data(), s.size()));
     require_tiledb_ok(vfs.close_file(ls_file));
-    auto&& [status, opt_children] = vfs.ls_with_sizes(ls_dir);
-    require_tiledb_ok(status);
-    auto children = opt_children.value();
+    auto children = vfs.ls_with_sizes(ls_dir);
 #ifdef _WIN32
     // Normalization only for Windows
     ls_file = URI(tiledb::sm::path_win::uri_from_path(ls_file.to_string()));
@@ -599,9 +597,7 @@ TEST_CASE("VFS: test ls_with_sizes", "[vfs][ls-with-sizes]") {
   require_tiledb_ok(vfs_ls.write(URI(subdir_file), s2.data(), s2.size()));
 
   // List
-  auto&& [status, rv] = vfs_ls.ls_with_sizes(URI(dir));
-  auto children = *rv;
-  require_tiledb_ok(status);
+  auto children = vfs_ls.ls_with_sizes(URI(dir));
 
 #ifdef _WIN32
   // Normalization only for Windows
@@ -713,37 +709,42 @@ TEST_CASE(
 
 #ifdef HAVE_AZURE
 TEST_CASE("VFS: Construct Azure Blob Storage endpoint URIs", "[azure][uri]") {
+  // Test the construction of Azure Blob Storage URIs from account name and SAS
+  // token. We are not actually connecting to Azure Blob Storage in this test.
   std::string sas_token, custom_endpoint, expected_endpoint;
   SECTION("No SAS token") {
     sas_token = "";
-    expected_endpoint = "https://devstoreaccount1.blob.core.windows.net";
+    expected_endpoint = "https://exampleaccount.blob.core.windows.net";
   }
   SECTION("SAS token without leading question mark") {
     sas_token = "baz=qux&foo=bar";
     expected_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
   }
   SECTION("SAS token with leading question mark") {
     sas_token = "?baz=qux&foo=bar";
     expected_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
   }
   SECTION("SAS token in both endpoint and config option") {
     sas_token = "baz=qux&foo=bar";
     custom_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
     expected_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
-  }
-  SECTION("No SAS token") {
-    sas_token = "";
-    expected_endpoint = "https://devstoreaccount1.blob.core.windows.net";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
   }
   Config config;
   require_tiledb_ok(
-      config.set("vfs.azure.storage_account_name", "devstoreaccount1"));
+      config.set("vfs.azure.storage_account_name", "exampleaccount"));
   require_tiledb_ok(config.set("vfs.azure.blob_endpoint", custom_endpoint));
   require_tiledb_ok(config.set("vfs.azure.storage_sas_token", sas_token));
+  if (sas_token.empty()) {
+    // If the SAS token is empty, the VFS will try to connect to Microsoft Entra
+    // ID to obtain credentials, which can take a long time because of retries.
+    // Set a dummy access key (which won't be used because we are not going to
+    // perform any requests) to prevent Entra ID from being chosen.
+    require_tiledb_ok(config.set("vfs.azure.storage_account_key", "foobar"));
+  }
   tiledb::sm::Azure azure;
   ThreadPool thread_pool(1);
   require_tiledb_ok(azure.init(config, &thread_pool));
