@@ -30,6 +30,6 @@
 
 int main() {
   std::string name("foo");
-  tiledb_dimension_handle_t x{name, tiledb::sm::Datatype::UINT32};
+  tiledb_dimension_handle_t x{name, tiledb::sm::Datatype::UINT32, nullptr};
   return 0;
 }
