@@ -29,7 +29,6 @@
 #include "../domain_api_internal.h"
 
 int main() {
-  // Domain is easy to deal with because it has a default constructor.
-  tiledb_domain_handle_t x{};
+  tiledb_domain_handle_t x{shared_ptr<tiledb::sm::Domain>()};
   return 0;
 }
