@@ -39,6 +39,8 @@
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/array_schema/array_schema_evolution.h"
 #include "tiledb/sm/array_schema/attribute.h"
+#include "tiledb/sm/array_schema/auxiliary.h"
+#include "tiledb/sm/array_schema/current_domain.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/crypto/crypto.h"
@@ -51,10 +53,11 @@
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_time.h"
 #include "tiledb/sm/misc/utils.h"
+#include "tiledb/sm/object/object.h"
+#include "tiledb/sm/object/object_mutex.h"
 #include "tiledb/sm/query/deletes_and_updates/serialization.h"
 #include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/rest/rest_client.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 
 #include <cassert>
@@ -84,10 +87,11 @@ ConsistencyController& controller() {
 /* ********************************* */
 
 Array::Array(
+    ContextResources& resources,
     const URI& array_uri,
-    StorageManager* storage_manager,
     ConsistencyController& cc)
-    : array_uri_(array_uri)
+    : resources_(resources)
+    , array_uri_(array_uri)
     , array_uri_serialized_(array_uri)
     , is_open_(false)
     , is_opening_or_closing_(false)
@@ -95,11 +99,9 @@ Array::Array(
     , user_set_timestamp_end_(nullopt)
     , array_dir_timestamp_end_(UINT64_MAX)
     , new_component_timestamp_(nullopt)
-    , storage_manager_(storage_manager)
-    , resources_(storage_manager_->resources())
     , config_(resources_.config())
     , remote_(array_uri.is_tiledb())
-    , memory_tracker_(storage_manager->resources().create_memory_tracker())
+    , memory_tracker_(resources_.create_memory_tracker())
     , consistency_controller_(cc)
     , consistency_sentry_(nullopt) {
 }
@@ -157,11 +159,75 @@ OpenedArray::load_delete_and_update_conditions() {
         throw_if_not_ok(conditions[i].check(array_schema_latest()));
         return Status::Ok();
       });
-  RETURN_NOT_OK_TUPLE(status, nullopt, nullopt);
+  throw_if_not_ok(status);
 
   return {Status::Ok(), conditions, update_values};
 }
 
+void Array::evolve_array_schema(
+    ContextResources& resources,
+    const URI& array_uri,
+    ArraySchemaEvolution* schema_evolution,
+    const EncryptionKey& encryption_key) {
+  // Check array schema
+  if (schema_evolution == nullptr) {
+    throw ArrayException("Cannot evolve array; Empty schema evolution");
+  }
+
+  if (array_uri.is_tiledb()) {
+    throw_if_not_ok(
+        resources.rest_client()->post_array_schema_evolution_to_rest(
+            array_uri, schema_evolution));
+    return;
+  }
+
+  // Load URIs from the array directory
+  tiledb::sm::ArrayDirectory array_dir{
+      resources,
+      array_uri,
+      0,
+      UINT64_MAX,
+      tiledb::sm::ArrayDirectoryMode::SCHEMA_ONLY};
+
+  // Check if array exists
+  if (!is_array(resources, array_uri)) {
+    throw ArrayException(
+        "Cannot evolve array; '" + array_uri.to_string() + "' does not exist");
+  }
+
+  auto&& array_schema = array_dir.load_array_schema_latest(
+      encryption_key, resources.ephemeral_memory_tracker());
+
+  // Load required enumerations before evolution.
+  auto enmr_names = schema_evolution->enumeration_names_to_extend();
+  if (enmr_names.size() > 0) {
+    std::unordered_set<std::string> enmr_path_set;
+    for (auto name : enmr_names) {
+      enmr_path_set.insert(array_schema->get_enumeration_path_name(name));
+    }
+    std::vector<std::string> enmr_paths;
+    for (auto path : enmr_path_set) {
+      enmr_paths.emplace_back(path);
+    }
+
+    auto loaded_enmrs = array_dir.load_enumerations_from_paths(
+        enmr_paths, encryption_key, resources.create_memory_tracker());
+
+    for (auto enmr : loaded_enmrs) {
+      array_schema->store_enumeration(enmr);
+    }
+  }
+
+  // Evolve schema
+  auto array_schema_evolved = schema_evolution->evolve_schema(array_schema);
+  try {
+    store_array_schema(resources, array_schema_evolved, encryption_key);
+  } catch (...) {
+    std::throw_with_nested(ArrayException(
+        "Cannot evolve schema; Not able to store evolved array schema."));
+  }
+}
+
 const URI& Array::array_uri() const {
   return array_uri_;
 }
@@ -174,6 +240,106 @@ const EncryptionKey* Array::encryption_key() const {
   return opened_array_->encryption_key();
 }
 
+void Array::create(
+    ContextResources& resources,
+    const URI& array_uri,
+    const shared_ptr<ArraySchema>& array_schema,
+    const EncryptionKey& encryption_key) {
+  // Check array schema
+  if (array_schema == nullptr) {
+    throw ArrayException("Cannot create array; Empty array schema");
+  }
+
+  // Check if array exists
+  if (is_array(resources, array_uri)) {
+    throw ArrayException(
+        "Cannot create array; Array '" + array_uri.to_string() +
+        "' already exists");
+  }
+
+  std::lock_guard<std::mutex> lock{object_mtx};
+  array_schema->set_array_uri(array_uri);
+  array_schema->generate_uri();
+  array_schema->check(resources.config());
+
+  // Check current domain is specified correctly if set
+  if (!array_schema->get_current_domain()->empty()) {
+    array_schema->get_current_domain()->check_schema_sanity(
+        array_schema->shared_domain());
+  }
+
+  // Create array directory
+  throw_if_not_ok(resources.vfs().create_dir(array_uri));
+
+  // Create array schema directory
+  URI array_schema_dir_uri =
+      array_uri.join_path(constants::array_schema_dir_name);
+  throw_if_not_ok(resources.vfs().create_dir(array_schema_dir_uri));
+
+  // Create the enumerations directory inside the array schema directory
+  URI array_enumerations_uri =
+      array_schema_dir_uri.join_path(constants::array_enumerations_dir_name);
+  throw_if_not_ok(resources.vfs().create_dir(array_enumerations_uri));
+
+  // Create commit directory
+  URI array_commit_uri = array_uri.join_path(constants::array_commits_dir_name);
+  throw_if_not_ok(resources.vfs().create_dir(array_commit_uri));
+
+  // Create fragments directory
+  URI array_fragments_uri =
+      array_uri.join_path(constants::array_fragments_dir_name);
+  throw_if_not_ok(resources.vfs().create_dir(array_fragments_uri));
+
+  // Create array metadata directory
+  URI array_metadata_uri =
+      array_uri.join_path(constants::array_metadata_dir_name);
+  throw_if_not_ok(resources.vfs().create_dir(array_metadata_uri));
+
+  // Create fragment metadata directory
+  URI array_fragment_metadata_uri =
+      array_uri.join_path(constants::array_fragment_meta_dir_name);
+  throw_if_not_ok(resources.vfs().create_dir(array_fragment_metadata_uri));
+
+  // Create dimension label directory
+  URI array_dimension_labels_uri =
+      array_uri.join_path(constants::array_dimension_labels_dir_name);
+  throw_if_not_ok(resources.vfs().create_dir(array_dimension_labels_uri));
+
+  // Store the array schema
+  try {
+    // Get encryption key from config
+    if (encryption_key.encryption_type() == EncryptionType::NO_ENCRYPTION) {
+      bool found = false;
+      std::string encryption_key_from_cfg =
+          resources.config().get("sm.encryption_key", &found);
+      assert(found);
+      std::string encryption_type_from_cfg =
+          resources.config().get("sm.encryption_type", &found);
+      assert(found);
+      auto&& [st_enc, etc] = encryption_type_enum(encryption_type_from_cfg);
+      throw_if_not_ok(st_enc);
+      EncryptionType encryption_type_cfg = etc.value();
+
+      EncryptionKey encryption_key_cfg;
+      if (encryption_key_from_cfg.empty()) {
+        throw_if_not_ok(
+            encryption_key_cfg.set_key(encryption_type_cfg, nullptr, 0));
+      } else {
+        throw_if_not_ok(encryption_key_cfg.set_key(
+            encryption_type_cfg,
+            (const void*)encryption_key_from_cfg.c_str(),
+            static_cast<uint32_t>(encryption_key_from_cfg.size())));
+      }
+      store_array_schema(resources, array_schema, encryption_key_cfg);
+    } else {
+      store_array_schema(resources, array_schema, encryption_key);
+    }
+  } catch (...) {
+    throw_if_not_ok(resources.vfs().remove_dir(array_uri));
+    throw;
+  }
+}
+
 // Used in Consolidator
 Status Array::open_without_fragments(
     EncryptionType encryption_type,
@@ -217,7 +383,7 @@ Status Array::open_without_fragments(
     if (remote_) {
       auto rest_client = resources_.rest_client();
       if (rest_client == nullptr) {
-        throw Status_ArrayError(
+        throw ArrayException(
             "Cannot open array; remote array with no REST client.");
       }
       /* #TODO Change get_array_schema_from_rest function signature to
@@ -421,12 +587,11 @@ Status Array::open(
             array_dir_timestamp_end_,
             ArrayDirectoryMode::SCHEMA_ONLY));
       }
-      auto&& [st, array_schema_latest, array_schemas] = open_for_writes();
-      throw_if_not_ok(st);
 
       // Set schemas
-      set_array_schema_latest(array_schema_latest.value());
-      set_array_schemas_all(std::move(array_schemas.value()));
+      auto&& [array_schema_latest, array_schemas] = open_for_writes();
+      set_array_schema_latest(array_schema_latest);
+      set_array_schemas_all(std::move(array_schemas));
 
       // Set the timestamp
       opened_array_->metadata().reset(timestamp_end_opened_at());
@@ -442,12 +607,11 @@ Status Array::open(
             array_dir_timestamp_end_,
             ArrayDirectoryMode::READ));
       }
-      auto&& [st, latest, array_schemas] = open_for_writes();
-      throw_if_not_ok(st);
 
       // Set schemas
-      set_array_schema_latest(latest.value());
-      set_array_schemas_all(std::move(array_schemas.value()));
+      auto&& [latest, array_schemas] = open_for_writes();
+      set_array_schema_latest(latest);
+      set_array_schemas_all(std::move(array_schemas));
 
       auto version = array_schema_latest().version();
       if (query_type == QueryType::DELETE &&
@@ -508,7 +672,7 @@ Status Array::close() {
         set_metadata_loaded(true);
         auto rest_client = resources_.rest_client();
         if (rest_client == nullptr) {
-          throw Status_ArrayError(
+          throw ArrayException(
               "Error closing array; remote array with no REST client.");
         }
         throw_if_not_ok(rest_client->post_array_metadata_to_rest(
@@ -532,7 +696,7 @@ Status Array::close() {
     opened_array_.reset();
   } catch (std::exception& e) {
     is_opening_or_closing_ = false;
-    throw Status_ArrayError(e.what());
+    throw ArrayException(e.what());
   }
 
   is_opening_or_closing_ = false;
@@ -573,11 +737,11 @@ void Array::delete_fragments(
   auto vfs = &(resources.vfs());
   throw_if_not_ok(parallel_for(
       &resources.compute_tp(), 0, fragment_uris.size(), [&](size_t i) {
-        RETURN_NOT_OK(vfs->remove_dir(fragment_uris[i].uri_));
+        throw_if_not_ok(vfs->remove_dir(fragment_uris[i].uri_));
         bool is_file = false;
-        RETURN_NOT_OK(vfs->is_file(commit_uris_to_delete[i], &is_file));
+        throw_if_not_ok(vfs->is_file(commit_uris_to_delete[i], &is_file));
         if (is_file) {
-          RETURN_NOT_OK(vfs->remove_file(commit_uris_to_delete[i]));
+          throw_if_not_ok(vfs->remove_file(commit_uris_to_delete[i]));
         }
         return Status::Ok();
       }));
@@ -666,7 +830,7 @@ void Array::delete_fragments_list(const std::vector<URI>& fragment_uris) {
   }
 }
 
-Status Array::encryption_type(
+void Array::encryption_type(
     ContextResources& resources,
     const URI& uri,
     EncryptionType* encryption_type) {
@@ -692,8 +856,6 @@ Status Array::encryption_type(
   auto&& header = GenericTileIO::read_generic_tile_header(
       resources, array_dir->latest_array_schema_uri(), 0);
   *encryption_type = static_cast<EncryptionType>(header.encryption_type);
-
-  return Status::Ok();
 }
 
 shared_ptr<const Enumeration> Array::get_enumeration(
@@ -992,7 +1154,7 @@ Status Array::reopen(uint64_t timestamp_start, uint64_t timestamp_end) {
       set_array_closed();
     } catch (std::exception& e) {
       is_opening_or_closing_ = false;
-      throw Status_ArrayError(e.what());
+      throw ArrayException(e.what());
     }
     is_opening_or_closing_ = false;
 
@@ -1678,19 +1840,15 @@ Array::open_for_reads_without_fragments() {
 }
 
 tuple<
-    Status,
-    optional<shared_ptr<ArraySchema>>,
-    optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>>
+    shared_ptr<ArraySchema>,
+    std::unordered_map<std::string, shared_ptr<ArraySchema>>>
 Array::open_for_writes() {
   auto timer_se =
       resources_.stats().start_timer("array_open_write_load_schemas");
   // Checks
-  if (!resources_.vfs().supports_uri_scheme(array_uri_))
-    return {
-        resources_.logger()->status(Status_StorageManagerError(
-            "Cannot open array; URI scheme unsupported.")),
-        nullopt,
-        nullopt};
+  if (!resources_.vfs().supports_uri_scheme(array_uri_)) {
+    throw ArrayException("Cannot open array; URI scheme unsupported.");
+  }
 
   // Load array schemas
   auto&& [array_schema_latest, array_schemas_all] =
@@ -1703,31 +1861,21 @@ Array::open_for_writes() {
   auto version = array_schema_latest->version();
   if constexpr (is_experimental_build) {
     if (version != constants::format_version) {
-      std::stringstream err;
-      err << "Cannot open array for writes; Array format version (";
-      err << version;
-      err << ") is not the library format version (";
-      err << constants::format_version << ")";
-      return {
-          resources_.logger()->status(Status_StorageManagerError(err.str())),
-          nullopt,
-          nullopt};
+      throw ArrayException(
+          "Cannot open array for writes; Array format version (" +
+          std::to_string(version) + ") is not the library format version (" +
+          std::to_string(constants::format_version) + ")");
     }
   } else {
     if (version > constants::format_version) {
-      std::stringstream err;
-      err << "Cannot open array for writes; Array format version (";
-      err << version;
-      err << ") is newer than library format version (";
-      err << constants::format_version << ")";
-      return {
-          resources_.logger()->status(Status_StorageManagerError(err.str())),
-          nullopt,
-          nullopt};
+      throw ArrayException(
+          "Cannot open array for writes; Array format version (" +
+          std::to_string(version) + ") is newer than library format version (" +
+          std::to_string(constants::format_version) + ")");
     }
   }
 
-  return {Status::Ok(), array_schema_latest, array_schemas_all};
+  return {array_schema_latest, array_schemas_all};
 }
 
 void Array::clear_last_max_buffer_sizes() {
@@ -1960,6 +2108,80 @@ const ArrayDirectory& Array::load_array_directory() {
   return array_directory();
 }
 
+void Array::upgrade_version(
+    ContextResources& resources,
+    const URI& array_uri,
+    const Config& override_config) {
+  // Check if array exists
+  if (!is_array(resources, array_uri)) {
+    throw ArrayException(
+        "Cannot upgrade array; Array '" + array_uri.to_string() +
+        "' does not exist");
+  }
+
+  // Load URIs from the array directory
+  tiledb::sm::ArrayDirectory array_dir{
+      resources,
+      array_uri,
+      0,
+      UINT64_MAX,
+      tiledb::sm::ArrayDirectoryMode::SCHEMA_ONLY};
+
+  // Get encryption key from config
+  bool found = false;
+  std::string encryption_key_from_cfg =
+      override_config.get("sm.encryption_key", &found);
+  assert(found);
+  std::string encryption_type_from_cfg =
+      override_config.get("sm.encryption_type", &found);
+  assert(found);
+  auto [st1, etc] = encryption_type_enum(encryption_type_from_cfg);
+  throw_if_not_ok(st1);
+  EncryptionType encryption_type_cfg = etc.value();
+
+  EncryptionKey encryption_key_cfg;
+  if (encryption_key_from_cfg.empty()) {
+    throw_if_not_ok(
+        encryption_key_cfg.set_key(encryption_type_cfg, nullptr, 0));
+  } else {
+    throw_if_not_ok(encryption_key_cfg.set_key(
+        encryption_type_cfg,
+        (const void*)encryption_key_from_cfg.c_str(),
+        static_cast<uint32_t>(encryption_key_from_cfg.size())));
+  }
+
+  auto&& array_schema = array_dir.load_array_schema_latest(
+      encryption_key_cfg, resources.ephemeral_memory_tracker());
+
+  if (array_schema->version() < constants::format_version) {
+    array_schema->generate_uri();
+    array_schema->set_version(constants::format_version);
+
+    // Create array schema directory if necessary
+    URI array_schema_dir_uri =
+        array_uri.join_path(constants::array_schema_dir_name);
+    throw_if_not_ok(resources.vfs().create_dir(array_schema_dir_uri));
+
+    // Store array schema
+    store_array_schema(resources, array_schema, encryption_key_cfg);
+
+    // Create commit directory if necessary
+    URI array_commit_uri =
+        array_uri.join_path(constants::array_commits_dir_name);
+    throw_if_not_ok(resources.vfs().create_dir(array_commit_uri));
+
+    // Create fragments directory if necessary
+    URI array_fragments_uri =
+        array_uri.join_path(constants::array_fragments_dir_name);
+    throw_if_not_ok(resources.vfs().create_dir(array_fragments_uri));
+
+    // Create fragment metadata directory if necessary
+    URI array_fragment_metadata_uri =
+        array_uri.join_path(constants::array_fragment_meta_dir_name);
+    throw_if_not_ok(resources.vfs().create_dir(array_fragment_metadata_uri));
+  }
+}
+
 Status Array::compute_non_empty_domain() {
   // Keep the current opened array alive for the duration of this call.
   auto opened_array = opened_array_;
@@ -2085,12 +2307,10 @@ void Array::ensure_array_is_valid_for_delete(const URI& uri) {
 void ensure_supported_schema_version_for_read(format_version_t version) {
   // We do not allow reading from arrays written by newer version of TileDB
   if (version > constants::format_version) {
-    std::stringstream err;
-    err << "Cannot open array for reads; Array format version (";
-    err << version;
-    err << ") is newer than library format version (";
-    err << constants::format_version << ")";
-    throw Status_StorageManagerError(err.str());
+    throw ArrayException(
+        "Cannot open array for reads; Array format version (" +
+        std::to_string(version) + ") is newer than library format version (" +
+        std::to_string(constants::format_version) + ")");
   }
 }
 
