@@ -45,13 +45,13 @@
 #include "tiledb/sm/crypto/encryption_key.h"
 #include "tiledb/sm/fragment/fragment_info.h"
 #include "tiledb/sm/metadata/metadata.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 
 using namespace tiledb::common;
 
 namespace tiledb::sm {
 
 class ArraySchema;
+class ArraySchemaEvolution;
 class SchemaEvolution;
 class FragmentMetadata;
 class MemoryTracker;
@@ -115,6 +115,11 @@ class OpenedArray {
         encryption_key_->set_key(encryption_type, key_bytes, key_length));
   }
 
+  /** Returns the context resources. */
+  inline ContextResources& resources() {
+    return resources_;
+  }
+
   /** Returns the array directory object. */
   inline const ArrayDirectory& array_directory() const {
     return array_dir_.value();
@@ -301,8 +306,8 @@ class Array {
 
   /** Constructor. */
   Array(
+      ContextResources& resources,
       const URI& array_uri,
-      StorageManager* storage_manager,
       ConsistencyController& cc = controller());
 
   /** Destructor. */
@@ -363,6 +368,20 @@ class Array {
     opened_array_->set_array_schemas_all(std::move(all_schemas));
   }
 
+  /**
+   * Evolve a TileDB array schema and store its new schema.
+   *
+   * @param resources The context resources.
+   * @param array_uri The uri of the array whose schema is to be evolved.
+   * @param schema_evolution The schema evolution.
+   * @param encryption_key The encryption key to use.
+   */
+  static void evolve_array_schema(
+      ContextResources& resources,
+      const URI& array_uri,
+      ArraySchemaEvolution* array_schema,
+      const EncryptionKey& encryption_key);
+
   /** Returns the array URI. */
   const URI& array_uri() const;
 
@@ -370,6 +389,20 @@ class Array {
    * serialization in pre TileDB 2.4 */
   const URI& array_uri_serialized() const;
 
+  /**
+   * Creates a TileDB array, storing its schema.
+   *
+   * @param resources The context resources.
+   * @param array_uri The URI of the array to be created.
+   * @param array_schema The array schema.
+   * @param encryption_key The encryption key to use.
+   */
+  static void create(
+      ContextResources& resources,
+      const URI& array_uri,
+      const shared_ptr<ArraySchema>& array_schema,
+      const EncryptionKey& encryption_key);
+
   /**
    * Opens the array for reading at a timestamp retrieved from the config
    * or for writing.
@@ -543,9 +576,8 @@ class Array {
    * @param resources The context resources.
    * @param uri The URI of the array.
    * @param encryption_type Set to the encryption type.
-   * @return Status
    */
-  static Status encryption_type(
+  static void encryption_type(
       ContextResources& resources,
       const URI& uri,
       EncryptionType* encryption_type);
@@ -1015,11 +1047,26 @@ class Array {
     return resources_.rest_client();
   }
 
+  /**
+   * Upgrade a TileDB array to latest format version.
+   *
+   * @param resources The context resources.
+   * @param uri The URI of the array.
+   * @param config Configuration parameters for the upgrade
+   *     (`nullptr` means default, which will use the config associated with
+   *      this instance).
+   */
+  static void upgrade_version(
+      ContextResources& resources, const URI& uri, const Config& config);
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /** TileDB Context Resources. */
+  ContextResources& resources_;
+
   /** The opened array that can be used by queries. */
   shared_ptr<OpenedArray> opened_array_;
 
@@ -1079,12 +1126,6 @@ class Array {
    */
   optional<uint64_t> new_component_timestamp_;
 
-  /** TileDB storage manager. */
-  StorageManager* storage_manager_;
-
-  /** TileDB Context Resources. */
-  ContextResources& resources_;
-
   /** The array config. */
   Config config_;
 
@@ -1156,19 +1197,18 @@ class Array {
       std::unordered_map<std::string, shared_ptr<ArraySchema>>>
   open_for_reads_without_fragments();
 
-  /** Opens an array for writes.
+  /**
+   * Opens an array for writes.
    *
    * @param array The array to open.
-   * @return tuple of Status, latest ArraySchema and map of all array schemas
-   *        Status Ok on success, else error
+   * @return tuple of latest ArraySchema and map of all array schemas
    *        ArraySchema The array schema to be retrieved after the
    *          array is opened.
    *        ArraySchemaMap Map of all array schemas found keyed by name
    */
   tuple<
-      Status,
-      optional<shared_ptr<ArraySchema>>,
-      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>>
+      shared_ptr<ArraySchema>,
+      std::unordered_map<std::string, shared_ptr<ArraySchema>>>
   open_for_writes();
 
   /** Clears the cached max buffer sizes and subarray. */
