@@ -42,6 +42,7 @@
  * API sections
  */
 #include "tiledb/api/c_api/attribute/attribute_api_external_experimental.h"
+#include "tiledb/api/c_api/current_domain/current_domain_api_external_experimental.h"
 #include "tiledb/api/c_api/enumeration/enumeration_api_experimental.h"
 #include "tiledb/api/c_api/query_aggregate/query_aggregate_api_external_experimental.h"
 #include "tiledb/api/c_api/query_field/query_field_api_external_experimental.h"
@@ -295,6 +296,49 @@ TILEDB_EXPORT int32_t tiledb_array_schema_evolution_set_timestamp_range(
     uint64_t lo,
     uint64_t hi) TILEDB_NOEXCEPT;
 
+/**
+ * Expands the current domain during array schema evolution.
+ * TileDB will enforce that the new current domain is expanding
+ * on the current one and not contracting during `tiledb_array_evolve`.
+ *
+ * **Example:**
+ *
+ * @code{.c}
+ * tiledb_current_domain_t *new_domain;
+ * tiledb_current_domain_create(ctx, &new_domain);
+ * tiledb_ndrectangle_t *ndr;
+ * tiledb_ndrectangle_alloc(ctx, domain, &ndr);
+ *
+ * tiledb_range_t range;
+ * range.min = &expanded_min;
+ * range.min_size = sizeof(expanded_min);
+ * range.max = &expanded_max;
+ * range.max_size = sizeof(expanded_max);
+ * tiledb_ndrectangle_set_range_for_name(ctx, ndr, "dim1", &range);
+ * tiledb_ndrectangle_set_range_for_name(ctx, ndr, "dim2", &range);
+ *
+ * tiledb_current_domain_set_ndrectangle(new_domain, ndr);
+ *
+ * tiledb_array_schema_evolution_expand_current_domain(ctx,
+ *      array_schema_evolution, new_domain);
+ *
+ * tiledb_array_evolve(ctx, array_uri, array_schema_evolution);
+ *
+ * tiledb_ndrectangle_free(&ndr);
+ * tiledb_current_domain_free(&new_domain);
+ *
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param array_schema_evolution The schema evolution.
+ * @param expanded_domain The current domain we want to expand the schema to.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT capi_return_t tiledb_array_schema_evolution_expand_current_domain(
+    tiledb_ctx_t* ctx,
+    tiledb_array_schema_evolution_t* array_schema_evolution,
+    tiledb_current_domain_t* expanded_domain) TILEDB_NOEXCEPT;
+
 /* ********************************* */
 /*          ARRAY SCHEMA             */
 /* ********************************* */
@@ -354,6 +398,51 @@ TILEDB_EXPORT int32_t tiledb_array_schema_add_enumeration(
     tiledb_array_schema_t* array_schema,
     tiledb_enumeration_t* enumeration) TILEDB_NOEXCEPT;
 
+/**
+ * Sets the current domain on the array schema
+ *
+ * **Example:**
+ *
+ * @code{.c}
+ * tiledb_current_domain_t *current_domain;
+ * tiledb_current_domain_create(ctx, &current_domain);
+ * tiledb_array_schema_set_current_domain(ctx, array_schema, current_domain);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param array_schema The array schema.
+ * @param current_domain The current domain to set on the schema
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_array_schema_set_current_domain(
+    tiledb_ctx_t* ctx,
+    tiledb_array_schema_t* array_schema,
+    tiledb_current_domain_t* current_domain) TILEDB_NOEXCEPT;
+
+/**
+ * Gets the current domain set on the array schema or
+ * creates an empty current domain if none was set.
+ * It is the responsability of the caller to free the resources associated
+ * with the current domain when the handle isn't needed anymore.
+ *
+ * **Example:**
+ *
+ * @code{.c}
+ * tiledb_current_domain_t *current_domain;
+ * tiledb_array_schema_get_current_domain(ctx, array_schema, &current_domain);
+ * tiledb_current_domain_free(&current_domain);
+ * @endcode
+ *
+ * @param ctx The TileDB context.
+ * @param array_schema The array schema.
+ * @param current_domain The current domain set on the schema
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_array_schema_get_current_domain(
+    tiledb_ctx_t* ctx,
+    tiledb_array_schema_t* array_schema,
+    tiledb_current_domain_t** current_domain) TILEDB_NOEXCEPT;
+
 /* ********************************* */
 /*               ARRAY               */
 /* ********************************* */
