@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  * @copyright Copyright (c) 2016 MIT and Intel Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -212,7 +212,7 @@ int32_t tiledb_filestore_uri_import(
   // Sync up the fragment timestamp and metadata timestamp
   uint64_t time_now = tiledb_timestamp_now_ms();
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::WRITE,
       0,
@@ -274,16 +274,13 @@ int32_t tiledb_filestore_uri_import(
   auto buffer_size =
       get_buffer_size_from_config(context.resources().config(), tile_extent);
 
-  tiledb::sm::Query query(context.storage_manager(), array);
+  tiledb::sm::Query query(
+      context.resources(), context.storage_manager(), array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::GLOBAL_ORDER));
   std::vector<std::byte> buffer(buffer_size);
 
   tiledb::sm::Subarray subarray(
-      array.get(),
-      nullptr,
-      context.storage_manager()->logger(),
-      true,
-      context.storage_manager());
+      array.get(), nullptr, context.resources().logger(), true);
   // We need to get the right end boundary of the last space tile.
   // The last chunk either falls exactly on the end of the file
   // or it goes beyond the end of the file so that it's equal in size
@@ -300,14 +297,11 @@ int32_t tiledb_filestore_uri_import(
   query.set_subarray(subarray);
 
   auto tiledb_cloud_fix = [&](uint64_t start, uint64_t end) {
-    tiledb::sm::Query query(context.storage_manager(), array);
+    tiledb::sm::Query query(
+        context.resources(), context.storage_manager(), array);
     throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
     tiledb::sm::Subarray subarray_cloud_fix(
-        array.get(),
-        nullptr,
-        context.storage_manager()->logger(),
-        true,
-        context.storage_manager());
+        array.get(), nullptr, context.resources().logger(), true);
 
     uint64_t cloud_fix_range_arr[] = {start, end};
     tiledb::type::Range subarray_range_cloud_fix(
@@ -399,7 +393,7 @@ int32_t tiledb_filestore_uri_export(
   }
 
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::READ,
       tiledb::sm::EncryptionType::NO_ENCRYPTION,
@@ -431,17 +425,14 @@ int32_t tiledb_filestore_uri_export(
   do {
     uint64_t write_size = end_range - start_range + 1;
     tiledb::sm::Subarray subarray(
-        array.get(),
-        nullptr,
-        context.storage_manager()->logger(),
-        true,
-        context.storage_manager());
+        array.get(), nullptr, context.resources().logger(), true);
     uint64_t subarray_range_arr[] = {start_range, end_range};
     tiledb::type::Range subarray_range(
         static_cast<void*>(subarray_range_arr), sizeof(uint64_t) * 2);
     subarray.add_range(0, std::move(subarray_range));
 
-    tiledb::sm::Query query(context.storage_manager(), array);
+    tiledb::sm::Query query(
+        context.resources(), context.storage_manager(), array);
     throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
     query.set_subarray(subarray);
 
@@ -502,7 +493,7 @@ int32_t tiledb_filestore_buffer_import(
   // Sync up the fragment timestamp and metadata timestamp
   uint64_t time_now = tiledb_timestamp_now_ms();
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::WRITE,
       0,
@@ -544,15 +535,12 @@ int32_t tiledb_filestore_buffer_import(
       static_cast<uint32_t>(0),
       "");
 
-  tiledb::sm::Query query(context.storage_manager(), array);
+  tiledb::sm::Query query(
+      context.resources(), context.storage_manager(), array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
 
   tiledb::sm::Subarray subarray(
-      array.get(),
-      nullptr,
-      context.storage_manager()->logger(),
-      true,
-      context.storage_manager());
+      array.get(), nullptr, context.resources().logger(), true);
   uint64_t subarray_range_arr[] = {
       static_cast<uint64_t>(0), static_cast<uint64_t>(size - 1)};
   tiledb::type::Range subarray_range(
@@ -582,7 +570,7 @@ int32_t tiledb_filestore_buffer_export(
 
   tiledb::sm::Context& context = ctx->context();
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::READ,
       tiledb::sm::EncryptionType::NO_ENCRYPTION,
@@ -610,18 +598,15 @@ int32_t tiledb_filestore_buffer_export(
   }
 
   tiledb::sm::Subarray subarray(
-      array.get(),
-      nullptr,
-      context.storage_manager()->logger(),
-      true,
-      context.storage_manager());
+      array.get(), nullptr, context.resources().logger(), true);
   uint64_t subarray_range_arr[] = {
       static_cast<uint64_t>(offset), static_cast<uint64_t>(offset + size - 1)};
   tiledb::type::Range subarray_range(
       static_cast<void*>(subarray_range_arr), sizeof(uint64_t) * 2);
   subarray.add_range(0, std::move(subarray_range));
 
-  tiledb::sm::Query query(context.storage_manager(), array);
+  tiledb::sm::Query query(
+      context.resources(), context.storage_manager(), array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
   query.set_subarray(subarray);
   uint64_t size_tmp = size;
@@ -642,7 +627,7 @@ int32_t tiledb_filestore_size(
 
   tiledb::sm::Context& context = ctx->context();
   tiledb::sm::Array array(
-      tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      context.resources(), tiledb::sm::URI(filestore_array_uri));
 
   throw_if_not_ok(array.open(
       tiledb::sm::QueryType::READ,
