@@ -118,12 +118,13 @@ VFS::VFS(
 /*                API                */
 /* ********************************* */
 
-std::string VFS::abs_path(const std::string& path) {
+std::string VFS::abs_path(std::string_view path) {
   // workaround for older clang (llvm 3.5) compilers (issue #828)
-  std::string path_copy = path;
+  std::string path_copy(path);
 #ifdef _WIN32
   {
-    std::string norm_sep_path = path_win::slashes_to_backslashes(path);
+    std::string norm_sep_path =
+        path_win::slashes_to_backslashes(std::string(path));
     if (path_win::is_win_path(norm_sep_path))
       return path_win::uri_from_path(Win::abs_path(norm_sep_path));
     else if (URI::is_file(path))
@@ -164,7 +165,8 @@ Status VFS::create_dir(const URI& uri) const {
 #ifdef _WIN32
     return win_.create_dir(uri.to_path());
 #else
-    return posix_.create_dir(uri);
+    posix_.create_dir(uri);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -220,10 +222,8 @@ Status VFS::dir_size(const URI& dir_name, uint64_t* dir_size) const {
   do {
     auto uri = to_ls.front();
     to_ls.pop_front();
-    auto&& [st, children] = ls_with_sizes(uri);
-    RETURN_NOT_OK(st);
 
-    for (const auto& child : *children) {
+    for (const auto& child : ls_with_sizes(uri)) {
       if (!child.is_directory()) {
         *dir_size += child.file_size();
       } else {
@@ -240,7 +240,8 @@ Status VFS::touch(const URI& uri) const {
 #ifdef _WIN32
     return win_.touch(uri.to_path());
 #else
-    return posix_.touch(uri);
+    posix_.touch(uri);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -252,7 +253,8 @@ Status VFS::touch(const URI& uri) const {
   }
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.touch(uri);
+    s3().touch(uri);
+    return Status::Ok();
 #else
     throw BuiltWithout("S3");
 #endif
@@ -285,7 +287,8 @@ Status VFS::cancel_all_tasks() {
 Status VFS::create_bucket(const URI& uri) const {
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.create_bucket(uri);
+    s3().create_bucket(uri);
+    return Status::Ok();
 #else
     throw BuiltWithout("S3");
 #endif
@@ -310,7 +313,8 @@ Status VFS::create_bucket(const URI& uri) const {
 Status VFS::remove_bucket(const URI& uri) const {
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.remove_bucket(uri);
+    s3().remove_bucket(uri);
+    return Status::Ok();
 #else
     throw BuiltWithout("S3");
 #endif
@@ -335,7 +339,8 @@ Status VFS::remove_bucket(const URI& uri) const {
 Status VFS::empty_bucket(const URI& uri) const {
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.empty_bucket(uri);
+    s3().empty_bucket(uri);
+    return Status::Ok();
 #else
     throw BuiltWithout("S3");
 #endif
@@ -361,7 +366,8 @@ Status VFS::is_empty_bucket(
     const URI& uri, [[maybe_unused]] bool* is_empty) const {
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.is_empty_bucket(uri, is_empty);
+    *is_empty = s3().is_empty_bucket(uri);
+    return Status::Ok();
 #else
     throw BuiltWithout("S3");
 #endif
@@ -388,7 +394,7 @@ Status VFS::remove_dir(const URI& uri) const {
 #ifdef _WIN32
     return win_.remove_dir(uri.to_path());
 #else
-    return posix_.remove_dir(uri);
+    posix_.remove_dir(uri);
 #endif
   } else if (uri.is_hdfs()) {
 #ifdef HAVE_HDFS
@@ -398,7 +404,7 @@ Status VFS::remove_dir(const URI& uri) const {
 #endif
   } else if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.remove_dir(uri);
+    s3().remove_dir(uri);
 #else
     throw BuiltWithout("S3");
 #endif
@@ -419,15 +425,16 @@ Status VFS::remove_dir(const URI& uri) const {
   } else {
     throw UnsupportedURI(uri.to_string());
   }
+  return Status::Ok();
 }
 
 void VFS::remove_dirs(
     ThreadPool* compute_tp, const std::vector<URI>& uris) const {
   throw_if_not_ok(parallel_for(compute_tp, 0, uris.size(), [&](size_t i) {
     bool is_dir;
-    RETURN_NOT_OK(this->is_dir(uris[i], &is_dir));
+    throw_if_not_ok(this->is_dir(uris[i], &is_dir));
     if (is_dir) {
-      RETURN_NOT_OK(remove_dir(uris[i]));
+      throw_if_not_ok(remove_dir(uris[i]));
     }
     return Status::Ok();
   }));
@@ -438,7 +445,8 @@ Status VFS::remove_file(const URI& uri) const {
 #ifdef _WIN32
     return win_.remove_file(uri.to_path());
 #else
-    return posix_.remove_file(uri);
+    posix_.remove_file(uri);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -450,7 +458,7 @@ Status VFS::remove_file(const URI& uri) const {
   }
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.remove_object(uri);
+    return s3().remove_object(uri);
 #else
     throw BuiltWithout("S3");
 #endif
@@ -478,7 +486,7 @@ Status VFS::remove_file(const URI& uri) const {
 void VFS::remove_files(
     ThreadPool* compute_tp, const std::vector<URI>& uris) const {
   throw_if_not_ok(parallel_for(compute_tp, 0, uris.size(), [&](size_t i) {
-    RETURN_NOT_OK(remove_file(uris[i]));
+    throw_if_not_ok(remove_file(uris[i]));
     return Status::Ok();
   }));
 }
@@ -486,7 +494,7 @@ void VFS::remove_files(
 void VFS::remove_files(
     ThreadPool* compute_tp, const std::vector<TimestampedURI>& uris) const {
   throw_if_not_ok(parallel_for(compute_tp, 0, uris.size(), [&](size_t i) {
-    RETURN_NOT_OK(remove_file(uris[i].uri_));
+    throw_if_not_ok(remove_file(uris[i].uri_));
     return Status::Ok();
   }));
 }
@@ -523,7 +531,8 @@ Status VFS::file_size(const URI& uri, uint64_t* size) const {
 #ifdef _WIN32
     return win_.file_size(uri.to_path(), size);
 #else
-    return posix_.file_size(uri, size);
+    posix_.file_size(uri, size);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -535,7 +544,7 @@ Status VFS::file_size(const URI& uri, uint64_t* size) const {
   }
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.object_size(uri, size);
+    return s3().object_size(uri, size);
 #else
     throw BuiltWithout("S3");
 #endif
@@ -579,7 +588,7 @@ Status VFS::is_dir(const URI& uri, bool* is_dir) const {
   }
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.is_dir(uri, is_dir);
+    return s3().is_dir(uri, is_dir);
 #else
     *is_dir = false;
     throw BuiltWithout("S3");
@@ -628,7 +637,7 @@ Status VFS::is_file(const URI& uri, bool* is_file) const {
   }
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    RETURN_NOT_OK(s3_.is_object(uri, is_file));
+    RETURN_NOT_OK(s3().is_object(uri, is_file));
     return Status::Ok();
 #else
     *is_file = false;
@@ -661,7 +670,7 @@ Status VFS::is_file(const URI& uri, bool* is_file) const {
 Status VFS::is_bucket(const URI& uri, bool* is_bucket) const {
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    RETURN_NOT_OK(s3_.is_bucket(uri, is_bucket));
+    *is_bucket = s3().is_bucket(uri);
     return Status::Ok();
 #else
     *is_bucket = false;
@@ -692,90 +701,74 @@ Status VFS::is_bucket(const URI& uri, bool* is_bucket) const {
 
 Status VFS::ls(const URI& parent, std::vector<URI>* uris) const {
   stats_->add_counter("ls_num", 1);
-  auto&& [st, entries] = ls_with_sizes(parent);
-  RETURN_NOT_OK(st);
 
-  for (auto& fs : *entries) {
+  for (auto& fs : ls_with_sizes(parent)) {
     uris->emplace_back(fs.path().native());
   }
 
   return Status::Ok();
 }
 
-tuple<Status, optional<std::vector<directory_entry>>> VFS::ls_with_sizes(
-    const URI& parent) const {
+std::vector<directory_entry> VFS::ls_with_sizes(const URI& parent) const {
   // Noop if `parent` is not a directory, do not error out.
   // For S3, GCS and Azure, `ls` on a non-directory will just
   // return an empty `uris` vector.
   if (!(parent.is_s3() || parent.is_gcs() || parent.is_azure())) {
     bool flag = false;
-    RETURN_NOT_OK_TUPLE(this->is_dir(parent, &flag), nullopt);
+    throw_if_not_ok(this->is_dir(parent, &flag));
 
     if (!flag) {
-      return {Status::Ok(), std::vector<directory_entry>()};
+      return {};
     }
   }
 
-  optional<std::vector<directory_entry>> entries;
+  std::vector<directory_entry> entries;
   if (parent.is_file()) {
 #ifdef _WIN32
-    Status st;
-    std::tie(st, entries) = win_.ls_with_sizes(parent);
+    entries = win_.ls_with_sizes(parent);
 #else
-    Status st;
-    std::tie(st, entries) = posix_.ls_with_sizes(parent);
+    entries = posix_.ls_with_sizes(parent);
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_s3()) {
 #ifdef HAVE_S3
-    Status st;
-    std::tie(st, entries) = s3_.ls_with_sizes(parent);
+    entries = s3().ls_with_sizes(parent);
 #else
-    auto st = Status_VFSError("TileDB was built without S3 support");
+    throw BuiltWithout("S3");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_azure()) {
 #ifdef HAVE_AZURE
-    Status st;
-    std::tie(st, entries) = azure_.ls_with_sizes(parent);
+    entries = azure_.ls_with_sizes(parent);
 #else
-    auto st = Status_VFSError("TileDB was built without Azure support");
+    throw BuiltWithout("Azure");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_gcs()) {
 #ifdef HAVE_GCS
-    Status st;
-    std::tie(st, entries) = gcs_.ls_with_sizes(parent);
+    entries = gcs_.ls_with_sizes(parent);
 #else
-    auto st = Status_VFSError("TileDB was built without GCS support");
+    throw BuiltWithout("GCS");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_hdfs()) {
 #ifdef HAVE_HDFS
-    Status st;
-    std::tie(st, entries) = hdfs_->ls_with_sizes(parent);
+    auto&& [st, entries_optional] = hdfs_->ls_with_sizes(parent);
+    throw_if_not_ok(st);
+    entries = *std::move(entries_optional);
 #else
-    auto st = Status_VFSError("TileDB was built without HDFS support");
+    throw BuiltWithout("HDFS");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_memfs()) {
-    Status st;
-    std::tie(st, entries) =
-        memfs_.ls_with_sizes(URI("mem://" + parent.to_path()));
-    RETURN_NOT_OK_TUPLE(st, nullopt);
+    entries = memfs_.ls_with_sizes(URI("mem://" + parent.to_path()));
   } else {
-    auto st = Status_VFSError("Unsupported URI scheme: " + parent.to_string());
-    return {st, std::nullopt};
+    throw UnsupportedURI(parent.to_string());
   }
   parallel_sort(
       compute_tp_,
-      entries->begin(),
-      entries->end(),
+      entries.begin(),
+      entries.end(),
       [](const directory_entry& l, const directory_entry& r) {
         return l.path().native() < r.path().native();
       });
 
-  return {Status::Ok(), entries};
+  return entries;
 }
 
 Status VFS::move_file(const URI& old_uri, const URI& new_uri) {
@@ -791,7 +784,8 @@ Status VFS::move_file(const URI& old_uri, const URI& new_uri) {
 #ifdef _WIN32
       return win_.move_path(old_uri.to_path(), new_uri.to_path());
 #else
-      return posix_.move_file(old_uri, new_uri);
+      posix_.move_file(old_uri, new_uri);
+      return Status::Ok();
 #endif
     }
     throw UnsupportedOperation("Moving files");
@@ -812,7 +806,7 @@ Status VFS::move_file(const URI& old_uri, const URI& new_uri) {
   if (old_uri.is_s3()) {
     if (new_uri.is_s3())
 #ifdef HAVE_S3
-      return s3_.move_object(old_uri, new_uri);
+      return s3().move_object(old_uri, new_uri);
 #else
       throw BuiltWithout("S3");
 #endif
@@ -860,7 +854,8 @@ Status VFS::move_dir(const URI& old_uri, const URI& new_uri) {
 #ifdef _WIN32
       return win_.move_path(old_uri.to_path(), new_uri.to_path());
 #else
-      return posix_.move_file(old_uri, new_uri);
+      posix_.move_dir(old_uri, new_uri);
+      return Status::Ok();
 #endif
     }
     throw UnsupportedOperation("Moving directories");
@@ -879,12 +874,14 @@ Status VFS::move_dir(const URI& old_uri, const URI& new_uri) {
 
   // S3
   if (old_uri.is_s3()) {
-    if (new_uri.is_s3())
+    if (new_uri.is_s3()) {
 #ifdef HAVE_S3
-      return s3_.move_dir(old_uri, new_uri);
+      s3().move_dir(old_uri, new_uri);
+      return Status::Ok();
 #else
       throw BuiltWithout("S3");
 #endif
+    }
     throw UnsupportedOperation("Moving directories");
   }
 
@@ -935,7 +932,8 @@ Status VFS::copy_file(const URI& old_uri, const URI& new_uri) {
 #ifdef _WIN32
       throw VFSException("Copying files on Windows is not yet supported.");
 #else
-      return posix_.copy_file(old_uri, new_uri);
+      posix_.copy_file(old_uri, new_uri);
+      return Status::Ok();
 #endif
     }
     throw UnsupportedOperation("Copying files");
@@ -955,12 +953,14 @@ Status VFS::copy_file(const URI& old_uri, const URI& new_uri) {
 
   // S3
   if (old_uri.is_s3()) {
-    if (new_uri.is_s3())
+    if (new_uri.is_s3()) {
 #ifdef HAVE_S3
-      return s3_.copy_file(old_uri, new_uri);
+      s3().copy_file(old_uri, new_uri);
+      return Status::Ok();
 #else
       throw BuiltWithout("S3");
 #endif
+    }
     throw UnsupportedOperation("Copying files");
   }
 
@@ -1000,7 +1000,8 @@ Status VFS::copy_dir(const URI& old_uri, const URI& new_uri) {
       throw VFSException(
           "Copying directories on Windows is not yet supported.");
 #else
-      return posix_.copy_dir(old_uri, new_uri);
+      posix_.copy_dir(old_uri, new_uri);
+      return Status::Ok();
 #endif
     }
     throw UnsupportedOperation("Copying directories");
@@ -1020,12 +1021,14 @@ Status VFS::copy_dir(const URI& old_uri, const URI& new_uri) {
 
   // S3
   if (old_uri.is_s3()) {
-    if (new_uri.is_s3())
+    if (new_uri.is_s3()) {
 #ifdef HAVE_S3
-      return s3_.copy_dir(old_uri, new_uri);
+      s3().copy_dir(old_uri, new_uri);
+      return Status::Ok();
 #else
       throw BuiltWithout("S3");
 #endif
+    }
     throw UnsupportedOperation("Copying directories");
   }
 
@@ -1135,7 +1138,8 @@ Status VFS::read_impl(
 #ifdef _WIN32
     return win_.read(uri.to_path(), offset, buffer, nbytes);
 #else
-    return posix_.read(uri, offset, buffer, nbytes, false);
+    posix_.read(uri, offset, buffer, nbytes, false);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -1148,8 +1152,8 @@ Status VFS::read_impl(
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     const auto read_fn = std::bind(
-        &S3::read,
-        &s3_,
+        &S3::read_impl,
+        &s3(),
         std::placeholders::_1,
         std::placeholders::_2,
         std::placeholders::_3,
@@ -1288,7 +1292,8 @@ Status VFS::sync(const URI& uri) {
 #ifdef _WIN32
     return win_.sync(uri.to_path());
 #else
-    return posix_.sync(uri);
+    posix_.sync(uri);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -1378,7 +1383,8 @@ Status VFS::close_file(const URI& uri) {
 #ifdef _WIN32
     return win_.sync(uri.to_path());
 #else
-    return posix_.sync(uri);
+    posix_.sync(uri);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -1390,7 +1396,7 @@ Status VFS::close_file(const URI& uri) {
   }
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    return s3_.flush_object(uri);
+    return s3().flush_object(uri);
 #else
     throw BuiltWithout("S3");
 #endif
@@ -1418,7 +1424,7 @@ Status VFS::close_file(const URI& uri) {
 void VFS::finalize_and_close_file(const URI& uri) {
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    s3_.finalize_and_flush_object(uri);
+    s3().finalize_and_flush_object(uri);
     return;
 #else
     throw BuiltWithout("S3");
@@ -1439,7 +1445,8 @@ Status VFS::write(
 #ifdef _WIN32
     return win_.write(uri.to_path(), buffer, buffer_size);
 #else
-    return posix_.write(uri, buffer, buffer_size);
+    posix_.write(uri, buffer, buffer_size);
+    return Status::Ok();
 #endif
   }
   if (uri.is_hdfs()) {
@@ -1451,11 +1458,8 @@ Status VFS::write(
   }
   if (uri.is_s3()) {
 #ifdef HAVE_S3
-    if (remote_global_order_write) {
-      s3_.global_order_write_buffered(uri, buffer, buffer_size);
-      return Status::Ok();
-    }
-    return s3_.write(uri, buffer, buffer_size);
+    s3().write(uri, buffer, buffer_size, remote_global_order_write);
+    return Status::Ok();
 #else
     throw BuiltWithout("S3");
 #endif
@@ -1487,7 +1491,7 @@ VFS::multipart_upload_state(const URI& uri) {
   } else if (uri.is_s3()) {
 #ifdef HAVE_S3
     VFS::MultiPartUploadState state;
-    auto s3_state = s3_.multipart_upload_state(uri);
+    auto s3_state = s3().multipart_upload_state(uri);
     if (!s3_state.has_value()) {
       return {Status::Ok(), nullopt};
     }
@@ -1553,11 +1557,11 @@ Status VFS::set_multipart_upload_state(
         // Chunk URI gets reconstructed from the serialized chunk name
         // and the real attribute uri
         s3_state.buffered_chunks.emplace_back(
-            s3_.generate_chunk_uri(uri, chunk.uri).to_string(), chunk.size);
+            s3().generate_chunk_uri(uri, chunk.uri).to_string(), chunk.size);
       }
     }
 
-    return s3_.set_multipart_upload_state(uri.to_string(), s3_state);
+    return s3().set_multipart_upload_state(uri.to_string(), s3_state);
 #else
     throw BuiltWithout("S3");
 #endif
@@ -1582,8 +1586,8 @@ Status VFS::flush_multipart_file_buffer(const URI& uri) {
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     Buffer* buff = nullptr;
-    throw_if_not_ok(s3_.get_file_buffer(uri, &buff));
-    s3_.global_order_write(uri, buff->data(), buff->size());
+    throw_if_not_ok(s3().get_file_buffer(uri, &buff));
+    s3().global_order_write(uri, buff->data(), buff->size());
     buff->reset_size();
 
 #else
