@@ -269,13 +269,13 @@ Status FilterPipeline::filter_chunks_forward(
 
       f->init_compression_resource_pool(compute_tp->concurrency_level());
 
-      RETURN_NOT_OK(f->run_forward(
+      f->run_forward(
           tile,
           offsets_tile,
           &input_metadata,
           &input_data,
           &output_metadata,
-          &output_data));
+          &output_data);
 
       input_data.set_read_only(false);
       throw_if_not_ok(input_data.swap(output_data));
@@ -358,11 +358,11 @@ Status FilterPipeline::filter_chunks_forward(
     std::memcpy((char*)dest + dest_offset, &metadata_size, sizeof(uint32_t));
     dest_offset += sizeof(uint32_t);
     // Write the chunk metadata
-    RETURN_NOT_OK(
+    throw_if_not_ok(
         final_stage_output_metadata.copy_to((char*)dest + dest_offset));
     dest_offset += metadata_size;
     // Write the chunk data
-    RETURN_NOT_OK(final_stage_output_data.copy_to((char*)dest + dest_offset));
+    throw_if_not_ok(final_stage_output_data.copy_to((char*)dest + dest_offset));
     return Status::Ok();
   });
 
@@ -382,13 +382,13 @@ uint32_t FilterPipeline::max_chunk_size() const {
   return max_chunk_size_;
 }
 
-Status FilterPipeline::run_forward(
+void FilterPipeline::run_forward(
     stats::Stats* const writer_stats,
     WriterTile* const tile,
     WriterTile* const offsets_tile,
     ThreadPool* const compute_tp,
     bool use_chunking) const {
-  RETURN_NOT_OK(
+  throw_if_not_ok(
       tile ? Status::Ok() : Status_Error("invalid argument: null Tile*"));
 
   writer_stats->add_counter("write_filtered_byte_num", tile->size());
@@ -404,25 +404,28 @@ Status FilterPipeline::run_forward(
   // Get the chunk sizes for var size attributes.
   auto&& [st, chunk_offsets] =
       get_var_chunk_sizes(chunk_size, tile, offsets_tile);
-  RETURN_NOT_OK_ELSE(st, tile->filtered_buffer().clear());
+  if (!st.ok()) {
+    tile->filtered_buffer().clear();
+    throw_if_not_ok(st);
+  }
 
   // Run the filters over all the chunks and store the result in
   // 'filtered_buffer'.
-  RETURN_NOT_OK_ELSE(
-      filter_chunks_forward(
-          *tile,
-          offsets_tile,
-          chunk_size,
-          *chunk_offsets,
-          tile->filtered_buffer(),
-          compute_tp),
-      tile->filtered_buffer().clear());
+  st = filter_chunks_forward(
+      *tile,
+      offsets_tile,
+      chunk_size,
+      *chunk_offsets,
+      tile->filtered_buffer(),
+      compute_tp);
+  if (!st.ok()) {
+    tile->filtered_buffer().clear();
+    throw_if_not_ok(st);
+  }
 
   // The contents of 'buffer' have been filtered and stored
   // in 'filtered_buffer'. We can safely free 'buffer'.
   tile->clear_data();
-
-  return Status::Ok();
 }
 
 void FilterPipeline::run_reverse_generic_tile(
