@@ -77,7 +77,7 @@ Datatype XORFilter::output_datatype(tiledb::sm::Datatype input_type) const {
   }
 }
 
-Status XORFilter::run_forward(
+void XORFilter::run_forward(
     const WriterTile&,
     WriterTile* const,
     FilterBuffer* input_metadata,
@@ -105,25 +105,25 @@ Status XORFilter::run_forward(
           input_metadata, input, output_metadata, output);
     }
     default: {
-      return Status_FilterError(
+      throw FilterStatusException(
           "XORFilter::run_forward: datatype size cannot be converted to "
           "integer type.");
     }
   }
 
-  return Status_FilterError("XORFilter::run_forward: invalid datatype.");
+  throw FilterStatusException("XORFilter::run_forward: invalid datatype.");
 }
 
 template <
     typename T,
     typename std::enable_if<std::is_integral<T>::value>::type*>
-Status XORFilter::run_forward(
+void XORFilter::run_forward(
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
     FilterBuffer* output) const {
   // Output size does not change with this filter.
-  RETURN_NOT_OK(output->prepend_buffer(input->size()));
+  throw_if_not_ok(output->prepend_buffer(input->size()));
   Buffer* output_buf = output->buffer_ptr(0);
   assert(output_buf != nullptr);
 
@@ -131,18 +131,16 @@ Status XORFilter::run_forward(
   auto parts = input->buffers();
   auto num_parts = (uint32_t)parts.size();
   uint32_t metadata_size = sizeof(uint32_t) + num_parts * sizeof(uint32_t);
-  RETURN_NOT_OK(output_metadata->append_view(input_metadata));
-  RETURN_NOT_OK(output_metadata->prepend_buffer(metadata_size));
-  RETURN_NOT_OK(output_metadata->write(&num_parts, sizeof(uint32_t)));
+  throw_if_not_ok(output_metadata->append_view(input_metadata));
+  throw_if_not_ok(output_metadata->prepend_buffer(metadata_size));
+  throw_if_not_ok(output_metadata->write(&num_parts, sizeof(uint32_t)));
 
   // XOR all parts
   for (const auto& part : parts) {
     auto part_size = (uint32_t)part.size();
-    RETURN_NOT_OK(output_metadata->write(&part_size, sizeof(uint32_t)));
-    RETURN_NOT_OK(xor_part<T>(&part, output_buf));
+    throw_if_not_ok(output_metadata->write(&part_size, sizeof(uint32_t)));
+    throw_if_not_ok(xor_part<T>(&part, output_buf));
   }
-
-  return Status::Ok();
 }
 
 template <
@@ -285,4 +283,4 @@ XORFilter* XORFilter::clone_impl() const {
 }
 
 }  // namespace sm
-}  // namespace tiledb
\ No newline at end of file
+}  // namespace tiledb
