@@ -48,7 +48,6 @@
 #include "tiledb/sm/query/readers/result_tile.h"
 #include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/subarray/cell_slab.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/type/apply_with_type.h"
@@ -59,9 +58,9 @@ using namespace tiledb::sm::stats;
 
 namespace tiledb::sm {
 
-class ReaderStatusException : public StatusException {
+class ReaderException : public StatusException {
  public:
-  explicit ReaderStatusException(const std::string& message)
+  explicit ReaderException(const std::string& message)
       : StatusException("Reader", message) {
   }
 };
@@ -110,23 +109,18 @@ Reader::Reader(
     bool remote_query)
     : ReaderBase(stats, logger->clone("Reader", ++logger_id_), params) {
   // Sanity checks
-  if (storage_manager_ == nullptr) {
-    throw ReaderStatusException(
-        "Cannot initialize reader; Storage manager not set");
-  }
-
   if (!params.default_channel_aggregates().empty()) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Reader cannot process aggregates");
   }
 
   if (!params.skip_checks_serialization() && buffers_.empty()) {
-    throw ReaderStatusException("Cannot initialize reader; Buffers not set");
+    throw ReaderException("Cannot initialize reader; Buffers not set");
   }
 
   if (!params.skip_checks_serialization() && array_schema_.dense() &&
       !subarray_.is_set()) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Dense reads must have a subarray set");
   }
 
@@ -142,6 +136,66 @@ Reader::Reader(
   check_validity_buffer_sizes();
 }
 
+/* ********************************* */
+/*          STATIC FUNCTIONS         */
+/* ********************************* */
+
+template <class T>
+void Reader::compute_result_space_tiles(
+    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
+    const std::vector<std::vector<uint8_t>>& tile_coords,
+    const TileDomain<T>& array_tile_domain,
+    const std::vector<TileDomain<T>>& frag_tile_domains,
+    std::map<const T*, ResultSpaceTile<T>>& result_space_tiles,
+    shared_ptr<MemoryTracker> memory_tracker) {
+  auto fragment_num = (unsigned)frag_tile_domains.size();
+  auto dim_num = array_tile_domain.dim_num();
+  std::vector<T> start_coords;
+  const T* coords;
+  start_coords.resize(dim_num);
+
+  // For all tile coordinates
+  for (const auto& tc : tile_coords) {
+    coords = (T*)(&(tc[0]));
+    start_coords = array_tile_domain.start_coords(coords);
+
+    // Create result space tile and insert into the map
+    auto r =
+        result_space_tiles.emplace(coords, ResultSpaceTile<T>(memory_tracker));
+    auto& result_space_tile = r.first->second;
+    result_space_tile.set_start_coords(start_coords);
+
+    // Add fragment info to the result space tile
+    for (unsigned f = 0; f < fragment_num; ++f) {
+      // Check if the fragment overlaps with the space tile
+      if (!frag_tile_domains[f].in_tile_domain(coords))
+        continue;
+
+      // Check if any previous fragment covers this fragment
+      // for the tile identified by `coords`
+      bool covered = false;
+      for (unsigned j = 0; j < f; ++j) {
+        if (frag_tile_domains[j].covers(coords, frag_tile_domains[f])) {
+          covered = true;
+          break;
+        }
+      }
+
+      // Exclude this fragment from the space tile
+      if (covered)
+        continue;
+
+      // Include this fragment in the space tile
+      auto frag_domain = frag_tile_domains[f].domain_slice();
+      auto frag_idx = frag_tile_domains[f].id();
+      result_space_tile.append_frag_domain(frag_idx, frag_domain);
+      auto tile_idx = frag_tile_domains[f].tile_pos(coords);
+      result_space_tile.set_result_tile(
+          frag_idx, tile_idx, *fragment_metadata[frag_idx].get());
+    }
+  }
+}
+
 /* ****************************** */
 /*               API              */
 /* ****************************** */
@@ -521,7 +575,7 @@ Status Reader::compute_range_result_coords(
   auto status =
       parallel_for(&resources_.compute_tp(), 0, range_num, [&](uint64_t r) {
         // Compute overlapping coordinates per range
-        RETURN_NOT_OK(compute_range_result_coords(
+        throw_if_not_ok(compute_range_result_coords(
             subarray,
             r,
             result_tile_map,
@@ -531,12 +585,14 @@ Status Reader::compute_range_result_coords(
         // Dedup unless there is a single fragment or array schema allows
         // duplicates
         if (!single_fragment[r] && !allows_dups) {
-          RETURN_CANCEL_OR_ERROR(sort_result_coords(
+          throw_if_not_ok(sort_result_coords(
               range_result_coords[r].begin(),
               range_result_coords[r].end(),
               range_result_coords[r].size(),
               sort_layout));
-          RETURN_CANCEL_OR_ERROR(dedup_result_coords(range_result_coords[r]));
+          throw_if_cancelled();
+          throw_if_not_ok(dedup_result_coords(range_result_coords[r]));
+          throw_if_cancelled();
         }
 
         return Status::Ok();
@@ -618,13 +674,14 @@ Status Reader::compute_range_result_coords(
   std::vector<std::vector<ResultCoords>> range_result_coords_vec(fragment_num);
   auto status =
       parallel_for(&resources_.compute_tp(), 0, fragment_num, [&](uint32_t f) {
-        return compute_range_result_coords(
+        throw_if_not_ok(compute_range_result_coords(
             subarray,
             range_idx,
             f,
             result_tile_map,
             result_tiles,
-            range_result_coords_vec[f]);
+            range_result_coords_vec[f]));
+        return Status::Ok();
       });
   RETURN_NOT_OK(status);
 
@@ -1475,6 +1532,23 @@ Status Reader::process_tiles(
   return Status::Ok();
 }
 
+template <class T>
+void Reader::compute_result_space_tiles(
+    const Subarray& subarray,
+    const Subarray& partitioner_subarray,
+    std::map<const T*, ResultSpaceTile<T>>& result_space_tiles) const {
+  auto&& [array_tile_domain, frag_tile_domains] =
+      compute_tile_domains<T>(partitioner_subarray);
+
+  compute_result_space_tiles<T>(
+      fragment_metadata_,
+      subarray.tile_coords(),
+      array_tile_domain,
+      frag_tile_domains,
+      result_space_tiles,
+      query_memory_tracker_);
+}
+
 template <class T>
 Status Reader::compute_result_cell_slabs(
     const Subarray& subarray,
@@ -1822,7 +1896,7 @@ void Reader::init_read_state() {
   // Check subarray
   if (subarray_.layout() == Layout::GLOBAL_ORDER &&
       subarray_.range_num() != 1) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize read state; Multi-range subarrays do not support "
         "global order");
   }
@@ -1831,21 +1905,21 @@ void Reader::init_read_state() {
   bool found = false;
   uint64_t memory_budget = 0;
   if (!config_.get<uint64_t>("sm.memory_budget", &memory_budget, &found).ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   uint64_t memory_budget_var = 0;
   if (!config_.get<uint64_t>("sm.memory_budget_var", &memory_budget_var, &found)
            .ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   offsets_format_mode_ = config_.get("sm.var_offsets.mode", &found);
   assert(found);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Unsupported offsets"
         " format in configuration");
   }
@@ -1854,19 +1928,19 @@ void Reader::init_read_state() {
            .get<bool>(
                "sm.var_offsets.extra_element", &offsets_extra_element_, &found)
            .ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   if (!config_
            .get<uint32_t>("sm.var_offsets.bitsize", &offsets_bitsize_, &found)
            .ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Unsupported offsets"
         " bitsize in configuration");
   }
@@ -1901,14 +1975,14 @@ void Reader::init_read_state() {
         if (!read_state_.partitioner_
                  .set_result_budget(attr_name.c_str(), *buffer_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       } else {
         if (!read_state_.partitioner_
                  .set_result_budget_nullable(
                      attr_name.c_str(), *buffer_size, *buffer_validity_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       }
     } else {
@@ -1917,7 +1991,7 @@ void Reader::init_read_state() {
                  .set_result_budget(
                      attr_name.c_str(), *buffer_size, *buffer_var_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       } else {
         if (!read_state_.partitioner_
@@ -1927,7 +2001,7 @@ void Reader::init_read_state() {
                      *buffer_var_size,
                      *buffer_validity_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       }
     }
