@@ -37,6 +37,7 @@
 #include "tiledb/common/memory.h"
 #include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/array/array.h"
+#include "tiledb/sm/array_schema/current_domain.h"
 #include "tiledb/sm/array_schema/dimension_label.h"
 #include "tiledb/sm/enums/query_status.h"
 #include "tiledb/sm/enums/query_type.h"
@@ -82,11 +83,12 @@ static uint64_t get_effective_memory_budget(
 /* ****************************** */
 
 Query::Query(
+    ContextResources& resources,
     StorageManager* storage_manager,
     shared_ptr<Array> array,
     optional<std::string> fragment_name,
     optional<uint64_t> memory_budget)
-    : resources_(storage_manager->resources())
+    : resources_(resources)
     , stats_(resources_.stats().create_child("Query"))
     , logger_(resources_.logger()->clone("Query", ++logger_id_))
     , query_memory_tracker_(resources_.memory_tracker_manager().create_tracker(
@@ -710,6 +712,7 @@ void Query::init() {
       // Initialize the dimension label queries.
       dim_label_queries_ = tdb_unique_ptr<ArrayDimensionLabelQueries>(tdb_new(
           ArrayDimensionLabelQueries,
+          resources_,
           storage_manager_,
           array_,
           subarray_,
@@ -833,6 +836,37 @@ Status Query::process() {
     condition_->rewrite_enumeration_conditions(array_schema());
   }
 
+  if (type_ == QueryType::READ) {
+    auto cd = array_schema_->get_current_domain();
+    if (!cd->empty()) {
+      // See if any data was written outside of the current domain.
+      bool all_ned_contained_in_current_domain = true;
+      for (auto& meta : fragment_metadata_) {
+        if (!cd->includes(meta->non_empty_domain())) {
+          all_ned_contained_in_current_domain = false;
+        }
+      }
+
+      for (Domain::dimension_size_type d = 0; d < array_schema_->dim_num();
+           d++) {
+        if (subarray_.is_set(d)) {
+          // Make sure all ranges are contained in the current domain.
+          for (auto& range : subarray_.ranges_for_dim(d)) {
+            if (!cd->includes(d, range)) {
+              throw QueryException(
+                  "A range was set outside of the current domain.");
+            }
+          }
+        } else if (!all_ned_contained_in_current_domain) {
+          // Add ranges to make sure all data read is contained in the current
+          // domain.
+          auto range_copy = cd->ndrectangle()->get_range(d);
+          subarray_.add_range(d, std::move(range_copy));
+        }
+      }
+    }
+  }
+
   // Update query status.
   status_ = QueryStatus::INPROGRESS;
 
@@ -1231,6 +1265,17 @@ Status Query::set_offsets_buffer(
     // Check number of coordinates
     uint64_t coords_num =
         *buffer_offsets_size / constants::cell_var_offset_size;
+
+    const bool offsets_extra_element_ =
+        config_.get<bool>("sm.var_offsets.extra_element", Config::must_find);
+
+    if (offsets_extra_element_) {
+      // the offsets buffer has `ncoords + 1` elements so that each coordinate
+      // is given by `[offset[i], offset[i + 1])` instead of using the length
+      // to determine the last
+      --coords_num;
+    }
+
     if (coord_offsets_buffer_is_set_ &&
         coords_num != coords_info_.coords_num_ && name == offsets_buffer_name_)
       return logger_->status(Status_QueryError(
@@ -1766,6 +1811,7 @@ bool Query::is_aggregate(std::string output_field_name) const {
 
 Status Query::create_strategy(bool skip_checks_serialization) {
   auto params = StrategyParams(
+      resources_,
       array_->memory_tracker(),
       query_memory_tracker_,
       storage_manager_,
@@ -1834,6 +1880,18 @@ Status Query::create_strategy(bool skip_checks_serialization) {
       all_dense &= frag_md->dense();
     }
 
+    // We are going to deprecate dense arrays with sparse fragments in 2.27 but
+    // log a warning for now.
+    if (array_schema_->dense() && !all_dense) {
+      LOG_WARN(
+          "This dense array contains sparse fragments. Support for reading "
+          "sparse fragments in dense arrays will be removed in TileDB version "
+          "2.27 to be released in September 2024. To make sure this array "
+          "continues to work after an upgrade to version 2.27 or later, please "
+          "consolidate the sparse fragments using a TileDB version 2.26 or "
+          "earlier.");
+    }
+
     if (is_dimension_label_ordered_read_) {
       strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
           OrderedDimLabelReader,
