@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,46 +38,95 @@
 #include "tiledb/common/common.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/dimension.h"
-#include "tiledb/sm/array_schema/dynamic_array.h"
 #include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/query/iquery_strategy.h"
 #include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/readers/reader_base.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/tile_cell_slab_iter.h"
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class Array;
 
 /** Processes dense read queries. */
 class DenseReader : public ReaderBase, public IQueryStrategy {
   /**
-   * TileSubarrays class to store tile subarrays inside of a dynamic array with
-   * custom destructor.
+   * Class that stores the tile data for an internal loop of the reader.
    */
-  class TileSubarrays : public DynamicArray<Subarray> {
+  template <class DimType>
+  class IterationTileData {
    public:
-    TileSubarrays(uint64_t n)
-        : DynamicArray<Subarray>(
-              n,
-              tdb::allocator<Subarray>{},
-              Tag<DynamicArray<Subarray>::NullInitializer>{})
-        , n_(n) {
+    /* ********************************* */
+    /*     CONSTRUCTORS & DESTRUCTORS    */
+    /* ********************************* */
+    IterationTileData() = delete;
+
+    IterationTileData(
+        ThreadPool& compute_tp,
+        Subarray& subarray,
+        uint64_t t_start,
+        uint64_t t_end,
+        std::map<const DimType*, ResultSpaceTile<DimType>>&& result_space_tiles)
+        : t_start_(t_start)
+        , t_end_(t_end)
+        , tile_subarrays_(t_end - t_start, subarray.dim_num())
+        , result_space_tiles_(std::move(result_space_tiles)) {
+      auto& tile_coords = subarray.tile_coords();
+      throw_if_not_ok(
+          parallel_for(&compute_tp, 0, tile_subarrays_.size(), [&](uint64_t t) {
+            subarray.crop_to_tile(
+                tile_subarrays_[t],
+                (const DimType*)&tile_coords[t + t_start][0]);
+            return Status::Ok();
+          }));
+    };
+
+    DISABLE_COPY_AND_COPY_ASSIGN(IterationTileData);
+    DISABLE_MOVE_AND_MOVE_ASSIGN(IterationTileData);
+
+    /* ********************************* */
+    /*          PUBLIC METHODS           */
+    /* ********************************* */
+
+    /** Returns the tile start index. */
+    inline uint64_t t_start() {
+      return t_start_;
     }
 
-    ~TileSubarrays() {
-      // Delete the tile subarrays.
-      for (uint64_t i = 0; i < n_; i++) {
-        (*this)[i].~Subarray();
-      }
+    /** Returns the tile end index. */
+    inline uint64_t& t_end() {
+      return t_end_;
+    }
+
+    /** Returns the tile subarrays. */
+    inline DenseTileSubarray<DimType>& tile_subarray(uint64_t t) {
+      return tile_subarrays_[t - t_start_];
+    }
+
+    /** Returns the result space tiles. */
+    inline std::map<const DimType*, ResultSpaceTile<DimType>>&
+    result_space_tiles() {
+      return result_space_tiles_;
     }
 
    private:
-    uint64_t n_;
+    /* ********************************* */
+    /*         PRIVATE ATTRIBUTES        */
+    /* ********************************* */
+
+    /** Start tile to process. */
+    uint64_t t_start_;
+
+    /** End tile to process. */
+    uint64_t t_end_;
+
+    /** Tile subarrays. */
+    std::vector<DenseTileSubarray<DimType>> tile_subarrays_;
+
+    /** Result space tiles. */
+    std::map<const DimType*, ResultSpaceTile<DimType>> result_space_tiles_;
   };
 
  public:
@@ -190,18 +239,30 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const std::unordered_set<std::string>& condition_names);
 
   /**
-   * Compute the result space tiles to process on an iteration to respect the
-   * memory budget.
+   * Computes the result space tiles based on the current partition.
+   *
+   * @tparam T The domain datatype.
+   * @param t_start The start tile index in the tile coords.
+   * @param names The fields to process.
+   * @param condition_names The fields in the query condition.
+   * @param subarray The input subarray.
+   * @param tiles_cell_num The cell num for all tiles.
+   * @param array_tile_domain The array tile domain.
+   * @param frag_tile_domains The relevant fragments tile domains.
+   * @param iteration_tile_data The iteration data.
+   *
+   * @return wait_compute_task_before_read, result_space_tiles.
    */
   template <class DimType>
-  uint64_t compute_space_tiles_end(
+  tuple<bool, std::map<const DimType*, ResultSpaceTile<DimType>>>
+  compute_result_space_tiles(
+      const uint64_t t_start,
       const std::vector<std::string>& names,
       const std::unordered_set<std::string>& condition_names,
-      Subarray& subarray,
-      uint64_t t_start,
-      std::map<const DimType*, ResultSpaceTile<DimType>>& result_space_tiles,
-      const DynamicArray<Subarray>& tile_subarrays,
-      ThreadPool::Task& compute_task);
+      const Subarray& subarray,
+      const std::vector<uint64_t>& tiles_cell_num,
+      const TileDomain<DimType>& array_tile_domain,
+      const std::vector<TileDomain<DimType>>& frag_tile_domains) const;
 
   /** Compute the result tiles to load for a name. */
   template <class DimType>
@@ -209,24 +270,20 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const optional<std::string> name,
       const std::unordered_set<std::string>& condition_names,
       const Subarray& subarray,
-      const uint64_t t_start,
-      const uint64_t t_end,
-      std::map<const DimType*, ResultSpaceTile<DimType>>& result_space_tiles,
-      const DynamicArray<Subarray>& tile_subarrays) const;
+      shared_ptr<IterationTileData<DimType>> iteration_tile_data,
+      const std::vector<uint64_t>& tiles_cell_num) const;
 
   /** Apply the query condition. */
   template <class DimType, class OffType>
   Status apply_query_condition(
       ThreadPool::Task& compute_task,
       Subarray& subarray,
-      const uint64_t t_start,
-      const uint64_t t_end,
       const std::unordered_set<std::string>& condition_names,
       const std::vector<DimType>& tile_extents,
-      DynamicArray<Subarray>& tile_subarrays,
+      const std::vector<uint64_t>& tiles_cell_num,
       std::vector<uint64_t>& tile_offsets,
       const std::vector<RangeInfo<DimType>>& range_info,
-      std::map<const DimType*, ResultSpaceTile<DimType>>& result_space_tiles,
+      shared_ptr<IterationTileData<DimType>> iteration_tile_data,
       const uint64_t num_range_threads,
       std::vector<uint8_t>& qc_result);
 
@@ -246,15 +303,12 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const std::string& name,
       const std::vector<DimType>& tile_extents,
       const Subarray& subarray,
-      const uint64_t t_start,
-      const uint64_t t_end,
       const uint64_t subarray_start_cell,
       const uint64_t subarray_end_cell,
-      const DynamicArray<Subarray>& tile_subarrays,
       const std::vector<uint64_t>& tile_offsets,
       uint64_t& var_buffer_size,
       const std::vector<RangeInfo<DimType>>& range_info,
-      std::map<const DimType*, ResultSpaceTile<DimType>>& result_space_tiles,
+      shared_ptr<IterationTileData<DimType>> iteration_tile_data,
       const std::vector<uint8_t>& qc_result,
       const uint64_t num_range_threads);
 
@@ -277,14 +331,14 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
    *
    * @param name Name of the field to process.
    * @param rst Result space tile.
-   * @param tile_subarray Tile subarray.
+   * @param tile_cell_num Tile cell num.
    * @return If we can do the aggregation with the frag md or not.
    */
   template <class DimType>
   inline bool can_aggregate_tile_with_frag_md(
       const std::string& name,
       ResultSpaceTile<DimType>& rst,
-      const Subarray& tile_subarray) const {
+      const uint64_t tile_cell_num) const {
     if (array_schema_.is_dim(name)) {
       return false;
     }
@@ -304,8 +358,7 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
 
     // Make sure this tile isn't cropped by ranges and the fragment metadata has
     // tile metadata.
-    if (tile_subarray.cell_num() != rt.cell_num() ||
-        !frag_md->has_tile_metadata()) {
+    if (tile_cell_num != rt.cell_num() || !frag_md->has_tile_metadata()) {
       return false;
     }
 
@@ -329,12 +382,10 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const std::string& name,
       const std::vector<DimType>& tile_extents,
       const Subarray& subarray,
-      const uint64_t t_start,
-      const uint64_t t_end,
-      const DynamicArray<Subarray>& tile_subarrays,
+      const std::vector<uint64_t>& tiles_cell_num,
       const std::vector<uint64_t>& tile_offsets,
       const std::vector<RangeInfo<DimType>>& range_info,
-      std::map<const DimType*, ResultSpaceTile<DimType>>& result_space_tiles,
+      shared_ptr<IterationTileData<DimType>> iteration_tile_data,
       const std::vector<uint8_t>& qc_result,
       const uint64_t num_range_threads);
 
@@ -356,7 +407,7 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const std::vector<DimType>& tile_extents,
       ResultSpaceTile<DimType>& result_space_tile,
       const Subarray& subarray,
-      const Subarray& tile_subarray,
+      const DenseTileSubarray<DimType>& tile_subarray,
       const uint64_t global_cell_offset,
       const std::vector<RangeInfo<DimType>>& range_info,
       const std::vector<uint8_t>& qc_result,
@@ -370,7 +421,7 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const std::vector<DimType>& tile_extents,
       ResultSpaceTile<DimType>& result_space_tile,
       const Subarray& subarray,
-      const Subarray& tile_subarray,
+      const DenseTileSubarray<DimType>& tile_subarray,
       const uint64_t subarray_start_cell,
       const uint64_t global_cell_offset,
       std::vector<void*>& var_data,
@@ -386,7 +437,7 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const std::vector<DimType>& tile_extents,
       ResultSpaceTile<DimType>& result_space_tile,
       const Subarray& subarray,
-      const Subarray& tile_subarray,
+      const DenseTileSubarray<DimType>& tile_subarray,
       const uint64_t subarray_start_cell,
       const uint64_t global_cell_offset,
       std::vector<void*>& var_data,
@@ -403,7 +454,7 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       const std::vector<DimType>& tile_extents,
       ResultSpaceTile<DimType>& result_space_tile,
       const Subarray& subarray,
-      const Subarray& tile_subarray,
+      const DenseTileSubarray<DimType>& tile_subarray,
       const uint64_t global_cell_offset,
       const std::vector<RangeInfo<DimType>>& range_info,
       const std::vector<uint8_t>& qc_result,
@@ -550,7 +601,6 @@ class DenseReader : public ReaderBase, public IQueryStrategy {
       std::vector<uint64_t>& offsets) const;
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_DENSE_READER
