@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,6 +35,7 @@
 
 #include "../strategy_base.h"
 #include "tiledb/common/common.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/array_schema/tile_domain.h"
@@ -46,11 +47,9 @@
 #include "tiledb/sm/query/readers/result_cell_slab.h"
 #include "tiledb/sm/query/readers/result_space_tile.h"
 #include "tiledb/sm/query/writers/domain_buffer.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/subarray_partitioner.h"
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class Array;
 class ArraySchema;
@@ -165,17 +164,7 @@ class ReaderBase : public StrategyBase {
 
   /** Constructor. */
   ReaderBase(
-      stats::Stats* stats,
-      shared_ptr<Logger> logger,
-      StorageManager* storage_manager,
-      Array* array,
-      Config& config,
-      std::unordered_map<std::string, QueryBuffer>& buffers,
-      std::unordered_map<std::string, QueryBuffer>& aggregate_buffers,
-      Subarray& subarray,
-      Layout layout,
-      std::optional<QueryCondition>& condition,
-      DefaultChannelAggregates& default_channel_aggregates);
+      stats::Stats* stats, shared_ptr<Logger> logger, StrategyParams& params);
 
   /** Destructor. */
   ~ReaderBase() = default;
@@ -184,34 +173,6 @@ class ReaderBase : public StrategyBase {
   /*          STATIC FUNCTIONS         */
   /* ********************************* */
 
-  /**
-   * Computes a mapping (tile coordinates) -> (result space tile).
-   * The produced result space tiles will contain information only
-   * about fragments that will contribute results. Specifically, if
-   * a fragment is completely covered by a more recent fragment
-   * in a particular space tile, then it will certainly not contribute
-   * results and, thus, no information about that fragment is included
-   * in the space tile.
-   *
-   * @tparam T The datatype of the tile domains.
-   * @param domain The array domain
-   * @param tile_coords The unique coordinates of the tiles that intersect
-   *     a subarray.
-   * @param array_tile_domain The array tile domain.
-   * @param frag_tile_domains The tile domains of each fragment. These
-   *     are assumed to be ordered from the most recent to the oldest
-   *     fragment.
-   * @param result_space_tiles The result space tiles to be produced
-   *     by the function.
-   */
-  template <class T>
-  static void compute_result_space_tiles(
-      const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
-      const std::vector<std::vector<uint8_t>>& tile_coords,
-      const TileDomain<T>& array_tile_domain,
-      const std::vector<TileDomain<T>>& frag_tile_domains,
-      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles);
-
   /**
    * Computes the minimum and maximum indexes of tile chunks to process based on
    * the available threads.
@@ -265,6 +226,9 @@ class ReaderBase : public StrategyBase {
   /*       PROTECTED ATTRIBUTES        */
   /* ********************************* */
 
+  /** The query's memory tracker. */
+  shared_ptr<MemoryTracker> memory_tracker_;
+
   /** The query condition. */
   std::optional<QueryCondition>& condition_;
 
@@ -570,7 +534,7 @@ class ReaderBase : public StrategyBase {
    *     `ResultTile` instances in this vector.
    * @return Filtered data blocks.
    */
-  std::vector<FilteredData> read_attribute_tiles(
+  std::list<FilteredData> read_attribute_tiles(
       const std::vector<NameToLoad>& names,
       const std::vector<ResultTile*>& result_tiles) const;
 
@@ -586,7 +550,7 @@ class ReaderBase : public StrategyBase {
    *     `ResultTile` instances in this vector.
    * @return Filtered data blocks.
    */
-  std::vector<FilteredData> read_coordinate_tiles(
+  std::list<FilteredData> read_coordinate_tiles(
       const std::vector<std::string>& names,
       const std::vector<ResultTile*>& result_tiles) const;
 
@@ -603,7 +567,7 @@ class ReaderBase : public StrategyBase {
    * @param validity_only Is the field read for validity only.
    * @return Filtered data blocks.
    */
-  std::vector<FilteredData> read_tiles(
+  std::list<FilteredData> read_tiles(
       const std::vector<NameToLoad>& names,
       const std::vector<ResultTile*>& result_tiles) const;
 
@@ -665,21 +629,29 @@ class ReaderBase : public StrategyBase {
    * @return Tile size.
    */
   uint64_t get_attribute_tile_size(
-      const std::string& name, unsigned f, uint64_t t);
+      const std::string& name, unsigned f, uint64_t t) const;
 
   /**
-   * Computes the result space tiles based on the current partition.
+   * Get the on disk size of an attribute tile.
+   *
+   * @param name The attribute name.
+   * @param f The fragment idx.
+   * @param t The tile idx.
+   * @return Tile size.
+   */
+  uint64_t get_attribute_persisted_tile_size(
+      const std::string& name, unsigned f, uint64_t t) const;
+
+  /**
+   * Computes the tile domains based on the current partition.
    *
    * @tparam T The domain datatype.
-   * @param subarray The input subarray.
    * @param partitioner_subarray The partitioner subarray.
-   * @param result_space_tiles The result space tiles to be computed.
+   * @return array tile domain, fragments tile domains.
    */
   template <class T>
-  void compute_result_space_tiles(
-      const Subarray& subarray,
-      const Subarray& partitioner_subarray,
-      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles) const;
+  std::pair<TileDomain<T>, std::vector<TileDomain<T>>> compute_tile_domains(
+      const Subarray& partitioner_subarray) const;
 
   /** Returns `true` if the coordinates are included in the attributes. */
   bool has_coords() const;
@@ -808,7 +780,6 @@ class ReaderBase : public StrategyBase {
       std::vector<uint64_t>& frag_first_array_tile_idx);
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_READER_BASE_H
