@@ -51,9 +51,9 @@
 
 namespace tiledb::sm {
 
-class SparseIndexReaderBaseStatusException : public StatusException {
+class SparseIndexReaderBaseException : public StatusException {
  public:
-  explicit SparseIndexReaderBaseStatusException(const std::string& message)
+  explicit SparseIndexReaderBaseException(const std::string& message)
       : StatusException("SparseIndexReaderBase", message) {
   }
 };
@@ -78,14 +78,9 @@ SparseIndexReaderBase::SparseIndexReaderBase(
           buffers_.count(constants::delete_timestamps) != 0)
     , partial_tile_offsets_loading_(false) {
   // Sanity checks
-  if (storage_manager_ == nullptr) {
-    throw SparseIndexReaderBaseStatusException(
-        "Cannot initialize reader; Storage manager not set");
-  }
-
   if (!params.skip_checks_serialization() && buffers_.empty() &&
       aggregate_buffers_.empty()) {
-    throw SparseIndexReaderBaseStatusException(
+    throw SparseIndexReaderBaseException(
         "Cannot initialize reader; Buffers not set");
   }
 
@@ -96,7 +91,7 @@ SparseIndexReaderBase::SparseIndexReaderBase(
   offsets_format_mode_ =
       config_.get<std::string>("sm.var_offsets.mode", Config::must_find);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    throw SparseIndexReaderBaseStatusException(
+    throw SparseIndexReaderBaseException(
         "Cannot initialize reader; Unsupported offsets format in "
         "configuration");
   }
@@ -108,7 +103,7 @@ SparseIndexReaderBase::SparseIndexReaderBase(
   offsets_bitsize_ =
       config_.get<uint32_t>("sm.var_offsets.bitsize", Config::must_find);
   if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    throw SparseIndexReaderBaseStatusException(
+    throw SparseIndexReaderBaseException(
         "Cannot initialize reader; Unsupported offsets bitsize in "
         "configuration");
   }
@@ -389,7 +384,7 @@ Status SparseIndexReaderBase::load_initial_data() {
   if (subarray_.is_set()) {
     // At this point, full memory budget is available.
     if (!array_memory_tracker_->set_budget(memory_budget_.total_budget())) {
-      throw SparseIndexReaderBaseStatusException(
+      throw SparseIndexReaderBaseException(
           "Cannot set array memory budget, already over limit.");
     }
 
@@ -454,7 +449,7 @@ Status SparseIndexReaderBase::load_initial_data() {
   // Set a limit to the array memory.
   if (!array_memory_tracker_->set_budget(
           memory_budget_.total_budget() * memory_budget_.ratio_array_data())) {
-    throw SparseIndexReaderBaseStatusException(
+    throw SparseIndexReaderBaseException(
         "Cannot set array memory budget, already over limit.");
   }
 
@@ -678,7 +673,7 @@ void SparseIndexReaderBase::compute_tile_bitmaps(
           {
             auto timer_compute_results_count_sparse =
                 stats_->start_timer("compute_results_count_sparse");
-            RETURN_NOT_OK(rt->compute_results_count_sparse(
+            throw_if_not_ok(rt->compute_results_count_sparse(
                 dim_idx,
                 ranges_for_dim,
                 relevant_ranges,
@@ -738,7 +733,7 @@ void SparseIndexReaderBase::apply_query_condition(
             // Remove cells with partial overlap from the bitmap.
             QueryCondition::Params params(
                 query_memory_tracker_, *(frag_meta->array_schema().get()));
-            RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<BitmapType>(
+            throw_if_not_ok(partial_overlap_condition_.apply_sparse<BitmapType>(
                 params, *rt, rt->bitmap()));
             rt->count_cells();
           }
@@ -755,8 +750,9 @@ void SparseIndexReaderBase::apply_query_condition(
             // Remove cells deleted cells using the open timestamp.
             QueryCondition::Params params(
                 query_memory_tracker_, *(frag_meta->array_schema().get()));
-            RETURN_NOT_OK(delete_timestamps_condition_.apply_sparse<BitmapType>(
-                params, *rt, rt->post_dedup_bitmap()));
+            throw_if_not_ok(
+                delete_timestamps_condition_.apply_sparse<BitmapType>(
+                    params, *rt, rt->post_dedup_bitmap()));
             if (array_schema_.allows_dups()) {
               rt->count_cells();
             }
@@ -766,7 +762,7 @@ void SparseIndexReaderBase::apply_query_condition(
           if (condition_.has_value()) {
             QueryCondition::Params params(
                 query_memory_tracker_, *(frag_meta->array_schema().get()));
-            RETURN_NOT_OK(condition_->apply_sparse<BitmapType>(
+            throw_if_not_ok(condition_->apply_sparse<BitmapType>(
                 params, *rt, rt->post_dedup_bitmap()));
             if (array_schema_.allows_dups()) {
               rt->count_cells();
@@ -801,12 +797,12 @@ void SparseIndexReaderBase::apply_query_condition(
                       *(frag_meta->array_schema().get()));
                   if (!frag_meta->has_timestamps() ||
                       delete_timestamp > frag_meta->timestamp_range().second) {
-                    RETURN_NOT_OK(
+                    throw_if_not_ok(
                         delete_and_update_conditions_[i]
                             .apply_sparse<BitmapType>(
                                 params, *rt, rt->post_dedup_bitmap()));
                   } else {
-                    RETURN_NOT_OK(
+                    throw_if_not_ok(
                         timestamped_delete_and_update_conditions_[i]
                             .apply_sparse<BitmapType>(
                                 params, *rt, rt->post_dedup_bitmap()));
