@@ -48,7 +48,6 @@
 #include "tiledb/sm/query/hilbert_order.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/sm/tile/tile_metadata_generator.h"
 #include "tiledb/sm/tile/writer_tile_tuple.h"
@@ -60,9 +59,9 @@ using namespace tiledb::sm::stats;
 
 namespace tiledb::sm {
 
-class GlobalOrderWriterStatusException : public StatusException {
+class GlobalOrderWriterException : public StatusException {
  public:
-  explicit GlobalOrderWriterStatusException(const std::string& message)
+  explicit GlobalOrderWriterException(const std::string& message)
       : StatusException("GlobalOrderWriter", message) {
   }
 };
@@ -96,14 +95,14 @@ GlobalOrderWriter::GlobalOrderWriter(
     , current_fragment_size_(0) {
   // Check the layout is global order.
   if (layout_ != Layout::GLOBAL_ORDER) {
-    throw GlobalOrderWriterStatusException(
+    throw GlobalOrderWriterException(
         "Failed to initialize global order writer. Layout " +
         layout_str(layout_) + " is not global order.");
   }
 
   // Check no ordered attributes.
   if (array_schema_.has_ordered_attributes()) {
-    throw GlobalOrderWriterStatusException(
+    throw GlobalOrderWriterException(
         "Failed to initialize global order writer. Global order writes to "
         "ordered attributes are not yet supported.");
   }
@@ -186,8 +185,7 @@ Status GlobalOrderWriter::alloc_global_write_state() {
   // Alloc FragmentMetadata object
   global_write_state_->frag_meta_ = this->create_fragment_metadata();
   // Used in serialization when FragmentMetadata is built from ground up
-  global_write_state_->frag_meta_->set_context_resources(
-      &storage_manager_->resources());
+  global_write_state_->frag_meta_->set_context_resources(&resources_);
 
   return Status::Ok();
 }
@@ -797,7 +795,7 @@ Status GlobalOrderWriter::global_write() {
     frag_meta->set_num_tiles(new_num_tiles);
 
     if (new_num_tiles == 0) {
-      throw GlobalOrderWriterStatusException(
+      throw GlobalOrderWriterException(
           "Fragment size is too small to write a single tile");
     }
 
@@ -872,8 +870,8 @@ Status GlobalOrderWriter::prepare_full_tiles(
     auto buff_it = buffers_.begin();
     std::advance(buff_it, i);
     const auto& name = buff_it->first;
-    RETURN_CANCEL_OR_ERROR(
-        prepare_full_tiles(name, coord_dups, &tiles->at(name)));
+    throw_if_not_ok(prepare_full_tiles(name, coord_dups, &tiles->at(name)));
+    throw_if_cancelled();
     return Status::Ok();
   });
 
