@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2023 TileDB Inc.
+ * @copyright Copyright (c) 2023-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -95,11 +95,8 @@ tdb_unique_ptr<Array> QueryPlanFx::create_array(const URI uri) {
   throw_if_not_ok(key.set_key(EncryptionType::NO_ENCRYPTION, nullptr, 0));
 
   // Create the (empty) array on disk.
-  Status st = sm_->array_create(uri, schema, key);
-  if (!st.ok()) {
-    throw std::runtime_error("Could not create array.");
-  }
-  tdb_unique_ptr<Array> array(new Array{uri, sm_.get()});
+  Array::create(resources_, uri, schema, key);
+  tdb_unique_ptr<Array> array(new Array{resources_, uri});
 
   return array;
 }
@@ -129,7 +126,7 @@ TEST_CASE_METHOD(QueryPlanFx, "Query plan dump_json", "[query_plan][dump]") {
   REQUIRE(st.ok());
 
   shared_ptr<Array> array_shared = std::move(array);
-  Query query(sm_.get(), array_shared);
+  Query query(resources_, sm_.get(), array_shared);
   REQUIRE(query.set_layout(Layout::ROW_MAJOR).ok());
 
   stats::Stats stats("foo");
