@@ -50,7 +50,6 @@
 #include "tiledb/sm/serialization/array_directory.h"
 #include "tiledb/sm/serialization/array_schema.h"
 #include "tiledb/sm/serialization/fragment_metadata.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
@@ -311,8 +310,21 @@ void array_from_capnp(
           &resources,
           array->memory_tracker(),
           frag_meta_reader.getVersion());
-      throw_if_not_ok(fragment_metadata_from_capnp(
-          array->array_schema_latest_ptr(), frag_meta_reader, meta));
+
+      auto schema = array->array_schema_latest_ptr();
+      if (frag_meta_reader.hasArraySchemaName()) {
+        auto fragment_array_schema_name =
+            frag_meta_reader.getArraySchemaName().cStr();
+        schema = array->array_schemas_all().at(fragment_array_schema_name);
+      } else if (array->array_schemas_all().size() > 1) {
+        throw ArraySerializationException(
+            "Cannot deserialize fragment metadata without an array schema name "
+            "in the case of arrays with evolved schemas.");
+      }
+
+      // pass the right schema to deserialize fragment metadata
+      throw_if_not_ok(
+          fragment_metadata_from_capnp(schema, frag_meta_reader, meta));
       if (client_side) {
         meta->loaded_metadata()->set_rtree_loaded();
       }
