@@ -41,6 +41,7 @@
 #include "tiledb/sm/array/array.h"
 #include "tiledb/sm/array_schema/array_schema_evolution.h"
 #include "tiledb/sm/array_schema/attribute.h"
+#include "tiledb/sm/array_schema/current_domain.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/array_schema/enumeration.h"
@@ -53,6 +54,7 @@
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/serialization/array_schema.h"
+#include "tiledb/sm/serialization/current_domain.h"
 #include "tiledb/sm/serialization/enumeration.h"
 
 #include <set>
@@ -152,11 +154,19 @@ Status array_schema_evolution_to_capnp(
   timestamp_builder.set(0, timestamp_range.first);
   timestamp_builder.set(1, timestamp_range.second);
 
+  auto crd = array_schema_evolution->current_domain_to_expand();
+  if (crd != nullptr) {
+    auto current_domain_builder =
+        array_schema_evolution_builder->initCurrentDomainToExpand();
+    current_domain_to_capnp(crd, current_domain_builder);
+  }
+
   return Status::Ok();
 }
 
 tdb_unique_ptr<ArraySchemaEvolution> array_schema_evolution_from_capnp(
-    const capnp::ArraySchemaEvolution::Reader& evolution_reader) {
+    const capnp::ArraySchemaEvolution::Reader& evolution_reader,
+    shared_ptr<MemoryTracker> memory_tracker) {
   // Create attributes to add
   std::unordered_map<std::string, shared_ptr<Attribute>> attrs_to_add;
   auto attrs_to_add_reader = evolution_reader.getAttributesToAdd();
@@ -177,7 +187,7 @@ tdb_unique_ptr<ArraySchemaEvolution> array_schema_evolution_from_capnp(
   std::unordered_map<std::string, shared_ptr<const Enumeration>> enmrs_to_add;
   auto enmrs_to_add_reader = evolution_reader.getEnumerationsToAdd();
   for (auto enmr_reader : enmrs_to_add_reader) {
-    auto enmr = enumeration_from_capnp(enmr_reader);
+    auto enmr = enumeration_from_capnp(enmr_reader, memory_tracker);
     enmrs_to_add[enmr->name()] = enmr;
   }
 
@@ -186,7 +196,7 @@ tdb_unique_ptr<ArraySchemaEvolution> array_schema_evolution_from_capnp(
       enmrs_to_extend;
   auto enmrs_to_extend_reader = evolution_reader.getEnumerationsToExtend();
   for (auto enmr_reader : enmrs_to_extend_reader) {
-    auto enmr = enumeration_from_capnp(enmr_reader);
+    auto enmr = enumeration_from_capnp(enmr_reader, memory_tracker);
     enmrs_to_extend[enmr->name()] = enmr;
   }
 
@@ -206,6 +216,20 @@ tdb_unique_ptr<ArraySchemaEvolution> array_schema_evolution_from_capnp(
     ts_range = std::make_pair(timestamp_range[0], timestamp_range[1]);
   }
 
+  shared_ptr<CurrentDomain> crd;
+  if (evolution_reader.hasCurrentDomainToExpand()) {
+    auto current_domain_reader = evolution_reader.getCurrentDomainToExpand();
+    // There is no available ArraySchema Domain here, so we'll construct
+    // the CurrentDomain with a nullptr Domain, and we'll set it properly
+    // during ArraySchema::expand_current_domain. Currently there is no
+    // room to play around with this dangling domain because these evolution
+    // deserialization APIs are only used on the REST server immediately before
+    // the schema is evolved on disk. This way we avoid serializing the domain
+    // along with the NDRectangle.
+    crd = current_domain_from_capnp(
+        current_domain_reader, nullptr, memory_tracker);
+  }
+
   return tdb_unique_ptr<ArraySchemaEvolution>(tdb_new(
       ArraySchemaEvolution,
       attrs_to_add,
@@ -213,7 +237,9 @@ tdb_unique_ptr<ArraySchemaEvolution> array_schema_evolution_from_capnp(
       enmrs_to_add,
       enmrs_to_extend,
       enmrs_to_drop,
-      ts_range));
+      ts_range,
+      crd,
+      memory_tracker));
 }
 
 Status array_schema_evolution_serialize(
@@ -275,7 +301,8 @@ Status array_schema_evolution_serialize(
 Status array_schema_evolution_deserialize(
     ArraySchemaEvolution** array_schema_evolution,
     SerializationType serialize_type,
-    const Buffer& serialized_buffer) {
+    const Buffer& serialized_buffer,
+    shared_ptr<MemoryTracker> memory_tracker) {
   try {
     tdb_unique_ptr<ArraySchemaEvolution> decoded_array_schema_evolution =
         nullptr;
@@ -291,8 +318,8 @@ Status array_schema_evolution_deserialize(
             array_schema_evolution_builder);
         capnp::ArraySchemaEvolution::Reader array_schema_evolution_reader =
             array_schema_evolution_builder.asReader();
-        decoded_array_schema_evolution =
-            array_schema_evolution_from_capnp(array_schema_evolution_reader);
+        decoded_array_schema_evolution = array_schema_evolution_from_capnp(
+            array_schema_evolution_reader, memory_tracker);
         break;
       }
       case SerializationType::CAPNP: {
@@ -303,8 +330,8 @@ Status array_schema_evolution_deserialize(
             serialized_buffer.size() / sizeof(::capnp::word)));
         capnp::ArraySchemaEvolution::Reader array_schema_evolution_reader =
             reader.getRoot<capnp::ArraySchemaEvolution>();
-        decoded_array_schema_evolution =
-            array_schema_evolution_from_capnp(array_schema_evolution_reader);
+        decoded_array_schema_evolution = array_schema_evolution_from_capnp(
+            array_schema_evolution_reader, memory_tracker);
         break;
       }
       default: {
@@ -343,7 +370,10 @@ Status array_schema_evolution_serialize(
 }
 
 Status array_schema_evolution_deserialize(
-    ArraySchemaEvolution**, SerializationType, const Buffer&) {
+    ArraySchemaEvolution**,
+    SerializationType,
+    const Buffer&,
+    shared_ptr<MemoryTracker>) {
   return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
